/*
 * $Id: FilterStack.java,v 1.13 2006/05/15 14:24:49 suhrbier Exp $
 * Created on 02.08.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: FilterStack.java,v $
 * Revision 1.13  2006/05/15 14:24:49  suhrbier
 * *** empty log message ***
 *
 * Revision 1.12  2006/05/11 19:03:52  suhrbier
 * *** empty log message ***
 *
 * Revision 1.11  2005/10/10 11:20:27  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 * Revision 1.10  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.9  2005/08/15 13:24:46  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.8  2005/08/06 14:00:29  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.7  2005/08/06 13:58:35  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.6  2005/08/05 14:58:24  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.5  2005/08/04 23:28:49  suhrbier
 * Response komplett auf ProtocolFilter umgestellt.
 *
 * Revision 1.4  2005/08/04 11:21:11  suhrbier
 * update protocolfilter
 *
 * Revision 1.3  2005/08/04 10:10:36  suhrbier
 * Umstellung auf ProtocolFilter
 *
 * Revision 1.2  2005/08/02 16:07:36  suhrbier
 * TestCase HeaderFilter added
 *
 * Revision 1.1  2005/08/02 14:26:56  suhrbier
 * First version
 *
 */
package nbi.xml.sax;

import java.net.URI;
import java.util.Iterator;
import java.util.Stack;

import javax.xml.namespace.NamespaceContext;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.NamespaceSupport;

/**
 * This class provides an element Stack and helper functionality to support the
 * parsing process of SAX events.
 * The filter stack stores SAX events as {@link SAXElement FilterStackElements} and keeps track of namespace prefixes in a {@link NamespaceContext NamespaceContext}.
 * It offers combinable track modes specifying the parts of a SAX element stored on the stack:
 * <ul>
 * <li>{@link #TRACK_MODE_NONE TRACK_MODE_NONE} Nothing is stored.</li>
 * <li>{@link #TRACK_MODE_URI TRACK_MODE_URI} The URI is stored.</li>
 * <li>{@link #TRACK_MODE_LOCALNAME TRACK_MODE_LOCALNAME} The local name is stored.</li>
 * <li>{@link #TRACK_MODE_QNAME TRACK_MODE_QNAME} The qualified name is stored.</li>
 * <li>{@link #TRACK_MODE_ATTRIBUTES TRACK_MODE_ATTRIBUTES} The attributes are stored.</li>
 * <li>{@link #TRACK_MODE_CHARACTERS TRACK_MODE_CHARACTERS} The value stored.</li>
 * </ul>
 * 
 * To update the stack, the following push- and pop methods are provided allowing to be used with the method signature of the corresponding {@link ContentHandler ContentHandler} interface methods or {@link SAXElement FilterStackElements}.
 * <ul>
 * <li>{@link #push(String, String, String, Attributes) push(String, String, String, Attributes)}</li>
 * <li>{@link #push(SAXElement) push(SAXElement)}</li>
 * <li>{@link #pop(String, String, String) pop(String, String, String)}</li>
 * <li>{@link #pop() pop()}</li>
 * <li>{@link #characters(char[], int, int) characters(char[], int, int)}</li>
 * <li>{@link #pushPrefixMapping(String, String)} pushPrefixMapping(String, String)</li>
 * <li>{@link #popPrefixMapping(String) popPrefixMapping(String)}</li>
 * </ul>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.13 $
 * @since
 */
public class FilterStack {
    /** TRACK_MODE track nothing */
    public static final int TRACK_MODE_NONE = 0;

    /** TRACK_MODE track URI */
    public static final int TRACK_MODE_URI = 1;

    /** TRACK_MODE track local name */
    public static final int TRACK_MODE_LOCALNAME = 2;

    /** TRACK_MODE track qualified name */
    public static final int TRACK_MODE_QNAME = 4;

    /** TRACK_MODE track attributes */
    public static final int TRACK_MODE_ATTRIBUTES = 8;

    /** TRACK_MODE track characters */
    public static final int TRACK_MODE_CHARACTERS = 16;

    /** The element stack */
    private Stack elements;

    /** The namespace stack */
    private NamespaceSupport namespace;

    /**
     * Flag indicating that a new namespace context is pushed in
     * {@link #pushPrefixMapping(String, String) pushPrefixMapping(String, String)}.
     */
    private boolean namespaceContextPushed = false;

    /** Current tracking mode */
    private int trackMode;

    /**
     * Inits an empty stack instance.
     * Sets track mode to {@link FilterStack#TRACK_MODE_NONE TRACK_MODE_NONE}.
     */
    public FilterStack() {
        this.elements = new Stack();
        this.trackMode = TRACK_MODE_NONE;
        this.namespace = new NamespaceSupport();
    }

    /**
     * Inits a stack instance with a track mode.
     * 
     * @param trackMode
     *            track mode of the new stack instance.
     */
    public FilterStack(int trackMode) {
        this.elements = new Stack();
        this.trackMode = trackMode;
        this.namespace = new NamespaceSupport();
    }

    /**
     * Pushes an startElement event on the stack. The element parts are stored as indicated in the
     * current track mode.
     * Resets the {@link #namespaceContextPushed namespaceContextPushed} flag possibly set in {@link #pushPrefixMapping(String, String) pushPrefixMapping()}.
     * 
     * @param uri
     *            element's URI
     * @param localName
     *            element's local name
     * @param qName
     *            element's qualified name
     * @param atts
     *            element's attributes
     * @throws SAXException If the given uri string violates RFC 2396, see
     *             {@link URI#URI(String) URI(String)}.
     */
    public void push(String uri, String localName, String qName, Attributes atts)
            throws SAXException {
        // Reset the prefix context flag (acts like startElement event)
        if (namespaceContextPushed)
            namespaceContextPushed = false;
        SAXElement element = new SAXElement(
                (isTrackModeUri() ? uri : null),
                (isTrackModeLocalName() ? localName : null),
                (isTrackModeQName() ? qName : null),
                (isTrackModeAttributes() ? new AttributesImpl(atts) : null));
        elements.push(element);
    }

    /**
     * Pops an endElement event from the stack. The last element is popped from
     * the stack and returned.
     * 
     * @param uri
     *            element's URI.
     * @param localName
     *            element's local name.
     * @param qName
     *            element's qualified name.
     * @return The element popped from the stack.
     * @throws SAXException
     *             If the parameters do not match the last element on the stack.
     */
    public SAXElement pop(String uri, String localName, String qName)
            throws SAXException {
        SAXElement ret = pop();
        // verify if stack element matches endElement parameters
        if ((isTrackModeUri() && !ret.getURI().equals(uri))
                || (isTrackModeLocalName() && !ret.getLocalName().equals(
                        localName))
                || (isTrackModeQName() && !ret.getQName().equals(qName)))
            throw new SAXException("Unexpected element " + uri + "/"
                    + localName);
        return ret;
    }

    /**
     * Updates the stack according to the SAX event characters. If in track mode
     * {@link #TRACK_MODE_CHARACTERS TRACK_MODE_CHARACTERS}, the characters are appended within to the value of the last element pushed on the stack.
     * 
     * @param ch
     *            value buffer
     * @param start
     *            start index in value buffer
     * @param length
     *            valid bytes into the value buffer
     */
    public void characters(char[] ch, int start, int length)
    {
        if (isTrackModeCharacters())
            ((SAXElement) elements.peek()).appendCharacters(ch, start,
                    length);
    }

    /**
     * Updates the stack according to the SAX event startPrefixMapping. Declares
     * the given prefix mapping on the namespace and pushes a new namespace context if
     * necessary. Sets the {@link #namespaceContextPushed namespaceContextPushed} flag.
     * 
     * @param prefix
     *            prefix to declare
     * @param uri
     *            uri of the prefix to declare
     */
    public boolean pushPrefixMapping(String prefix, String uri) {
        if (!namespaceContextPushed)
        {
            namespace.pushContext();
            namespaceContextPushed = true;            
        }
        return namespace.declarePrefix(prefix, uri);
    }

    /**
     * Updates the stack according to the SAX event startPrefixMapping. Pops the
     * current context from the namespace.
     * 
     * @param prefix
     *            prefix to undeclare
     */
    public void popPrefixMapping(String prefix) {
        namespace.popContext();
    }

    /**
     * Gets the element stack.
     * 
     * @return Returns the element stack.
     */
    public Stack getElements() {
        return elements;
    }

    /**
     * Sets the element stack.
     * 
     * @param elements
     *            The treeElements stack to set.
     */
    public void setElements(Stack elements) {
        this.elements = elements;
    }

    /**
     * Gets the current namespace.
     * 
     * @return Returns the namespace.
     */
    public NamespaceSupport getNamespace() {
        return namespace;
    }

    /**
     * Sets the current namespace.
     * 
     * @param namespace
     *            The namespace to set.
     */
    public void setNamespace(NamespaceSupport namespace) {
        this.namespace = namespace;
    }

    /**
     * Evaluates if the stack is in track mode <code>none</code>.
     * 
     * @return true, if stack is in track mode <code>none</code>, false
     *         otherwise.
     */
    boolean isTrackModeNone() {
        return trackMode == TRACK_MODE_NONE;
    }

    /**
     * Evaluates if the stack is in track mode <code>URI</code>.
     * 
     * @return true, if stack is in track mode <code>URI</code>, false
     *         otherwise.
     */
    boolean isTrackModeUri() {
        return (TRACK_MODE_URI & trackMode) != 0;
    }

    /**
     * Evaluates if the stack is in track mode <code>local name</code>.
     * 
     * @return true, if stack is in track mode <code>local name</code>, false
     *         otherwise.
     */
    boolean isTrackModeLocalName() {
        return (TRACK_MODE_LOCALNAME & trackMode) != 0;
    }

    /**
     * Evaluates if the stack is in track mode <code>qualified name</code>.
     * 
     * @return true, if stack is in track mode <code>qualified name</code>,
     *         false otherwise.
     */
    boolean isTrackModeQName() {
        return (TRACK_MODE_QNAME & trackMode) != 0;
    }

    /**
     * Evaluates if the stack is in track mode <code>attributes</code>.
     * 
     * @return true, if stack is in track mode <code>attributes</code>, false
     *         otherwise.
     */
    boolean isTrackModeAttributes() {
        return (TRACK_MODE_ATTRIBUTES & trackMode) != 0;
    }

    /**
     * Evaluates if the stack is in track mode <code>characters</code>.
     * 
     * @return true, if stack is in track mode <code>characters</code>, false
     *         otherwise.
     */
    boolean isTrackModeCharacters() {
        return (TRACK_MODE_CHARACTERS & trackMode) != 0;
    }

    /**
     * Gets the current track mode.
     * 
     * @return Returns the trackMode.
     */
    public int getTrackMode() {
        return trackMode;
    }

    /**
     * Sets the current track mode.
     * 
     * @param trackMode
     *            The trackMode to set.
     */
    public void setTrackMode(int trackMode) {
        this.trackMode = trackMode;
    }

    /**
     * Peeks one element from the stack.
     * 
     * @return The peeked element.
     */
    public SAXElement peek() {
        return (SAXElement) elements.peek();
    }

    /**
     * Pops one element from the stack.
     * 
     * @return The popped element.
     */
    public SAXElement pop() {
        return (SAXElement) elements.pop();
    }

    /**
     * Pushes one element from the stack.
     * @param element element to push on the stack.
     * @return The pushed element.
     */
    public SAXElement push(SAXElement element) {
        return (SAXElement) elements.push(element);
    }

    /**
     * Tests if the stack is empty.
     * 
     * @return <code>true</code>, if the stack is empty, <code>false</code>
     *         otherwise.
     */
    public boolean empty() {
        return elements.empty();
    }

    /**
     * Builds the path to the current element on the element stack.
     * @param sep Path delimiter string.
     * @return element path.
     */
    public String buildElementPath(String sep) {
        StringBuffer elementPath = new StringBuffer();
        Iterator it = elements.iterator();
        while (it.hasNext())
            elementPath.append(sep
                    + ((SAXElement) it.next()).getLocalName());
        return elementPath.toString();
    }

}
