/*
 * $Id: PEP.java,v 1.17 2005/12/19 19:11:06 suhrbier Exp $
 * Created on 05.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PEP.java,v $
 * Revision 1.17  2005/12/19 19:11:06  suhrbier
 * Version ready to be delivered to BGBM.
 *
 * Revision 1.16  2005/12/15 19:41:57  suhrbier
 * Distribution is running
 *
 * Revision 1.15  2005/10/07 17:05:51  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 *
 */
package nbi.xmlsec;

import java.lang.reflect.InvocationTargetException;
import java.net.UnknownHostException;

import javax.net.ssl.SSLServerSocket;

import nbi.util.ReflectionUtil;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.ConfigurationFactory;
import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;
import org.mortbay.http.HttpContext;
import org.mortbay.http.HttpServer;
import org.mortbay.http.SslListener;

/**
 * This class implements a Policy Enforcement Point. The implementation bases on
 * the Jetty HttpServer implementation. The PEP acts like an Filter between a
 * client and a resource server. The configuration is done in the config.xml
 * file, according to the Jakarta Commons Configuration API.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.17 $
 * @since
 */
public class PEP {

    static Logger logger = Logger.getLogger(PEP.class);

    /** The PEP's configuration */
    private Configuration configuration;

    /** The PEP's filter */
    private HttpServer filter;

    /**
     * Inits the PEP based on the given configuration. Depending on the
     * configuration a Http-Server is set up creating appropriate listeners and
     * handlers.
     * 
     * @param config
     *            PEP's configuration.
     * @throws Exception
     */
    public PEP(Configuration config) throws Exception {
        configuration = config;
        filter = (HttpServer) getFilterInstance(config);
        filter.start();
        if (config.getBoolean("PEP.Filter.Listener.SSL[@wantClientAuth]"))
            ((SSLServerSocket) ((SslListener) filter.getListeners()[0])
                    .getServerSocket()).setWantClientAuth(true);
    }

    /**
     * Sets up the PEP filter based on the given configuration. Depending on the
     * configuration a Http-Server is set up creating appropriate listeners and
     * handlers.
     * 
     * @param config
     *            PEP's configuration.
     * @throws Exception
     */
    protected Object getFilterInstance(Configuration config)
            throws UnknownHostException, ClassNotFoundException,
            NoSuchMethodException, InvocationTargetException,
            InstantiationException, IllegalAccessException {
        HttpServer filter = (HttpServer) ReflectionUtil.getInstance(config
                .getString("PEP.Filter[@class]"));
        SslListener listener = (SslListener) ReflectionUtil.getInstance(config
                .getString("PEP.Filter.Listener[@class]"));
        listener.setHost(config.getString("PEP.Filter.Listener[@host]"));
        listener.setPort(config.getInt("PEP.Filter.Listener[@port]"));
        listener.setNeedClientAuth(config
                .getBoolean("PEP.Filter.Listener.SSL[@needClientAuth]"));
        listener.setKeystore(config
                .getString("PEP.Filter.Listener.SSL.KeyStore[@file]"));
        listener.setKeystoreType(config
                .getString("PEP.Filter.Listener.SSL.KeyStore[@type]"));
        listener.setKeyPassword(config
                .getString("PEP.Filter.Listener.SSL.KeyStore[@password]"));
        listener.setPassword(config
                .getString("PEP.Filter.Listener.SSL.Key[@password]"));
        filter.addListener(listener);

        HttpContext context = filter.getContext(config
                .getString("PEP.Filter.Context[@path]"));
        PEPHandler handler = (PEPHandler) ReflectionUtil.getInstance(config
                .getString("PEP.Filter.Handler[@class]"), Configuration.class,
                config.subset("PEP.Filter.Handler"));
        context.addHandler(handler);

        return filter;
    }

    /**
     * Starts the PEP. Sets the wantClientAuth option of the ServerSocket
     * according to the configuration.
     * 
     * @throws Exception
     */
    void start() throws Exception {
        filter.start();
        ((SSLServerSocket) ((SslListener) filter.getListeners()[0])
                .getServerSocket()).setWantClientAuth(configuration.getBoolean(
                "PEP.Filter.Listener.SSL[@wantClientAuth]", false));
    }

    /**
     * Stops the PEP.
     * 
     * @throws Exception
     */
    void stop() throws InterruptedException {
        filter.stop();
    }

    /**
     * Gets the configuration
     * 
     * @return Returns the configuration.
     */
    public Configuration getConfiguration() {
        return configuration;
    }

    /**
     * Sets configuration
     * 
     * @param configuration
     *            The configuration to set.
     */
    public void setConfiguration(Configuration configuration) {
        this.configuration = configuration;
    }

    /**
     * Sets filter
     * 
     * @param filter
     *            The filter to set.
     */
    public void setFilter(HttpServer filter) {
        this.filter = filter;
    }

    /**
     * Gets the filter
     * 
     * @return Returns the filter.
     */
    public HttpServer getFilter() {
        return filter;
    }

    /**
     * Get the PEP's class name from the configuration.
     * 
     * @param config
     *            PEP's configuration
     * @return PEP's class name or null.
     */
    static public String getClassName(Configuration config) {
        return config.getString("PEP[@class]");
    }

    static public void main(String args[]) throws Exception {
        // Shut down the java.util.logging used from sunxacml
        java.util.logging.LogManager.getLogManager().reset();

        DOMConfigurator.configure("log4j.xml");

        try {
            ConfigurationFactory factory = new ConfigurationFactory(
                    "config.xml");
            Configuration config = factory.getConfiguration();

            PEP pep = (PEP) ReflectionUtil.getInstance(
                    PEP.getClassName(config), Configuration.class, config);
            pep.start();

        } catch (ConfigurationException e) {
            logger.error(e.getMessage());
        }
    }
}
