/*
 * $Id: PEPClientKeySelector.java,v 1.6 2006/07/25 14:59:03 suhrbier Exp $
 * Created on 14.07.2006
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PEPClientKeySelector.java,v $
 * Revision 1.6  2006/07/25 14:59:03  suhrbier
 * update JavaDoc
 *
 * Revision 1.5  2006/07/24 17:31:09  suhrbier
 * update JavaDoc
 *
 * Revision 1.4  2006/07/24 08:36:52  suhrbier
 * KeySelectorException auf getCause umgestellt.
 *
 * Revision 1.3  2006/07/21 12:22:26  suhrbier
 * +CertPathValidation
 *
 * Revision 1.2  2006/07/19 10:34:14  suhrbier
 * + Keyinfo support certificates (not completed)
 *
 * Revision 1.1  2006/07/14 21:17:58  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 */
package nbi.xmlsec;

import java.io.FileInputStream;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.KeyException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import java.security.cert.CertPathBuilder;
import java.security.cert.CertPathBuilderException;
import java.security.cert.CertPathValidator;
import java.security.cert.CertPathValidatorException;
import java.security.cert.CertPathValidatorResult;
import java.security.cert.CertificateException;
import java.security.cert.PKIXBuilderParameters;
import java.security.cert.PKIXParameters;
import java.security.cert.X509CRL;
import java.security.cert.X509CertSelector;
import java.security.cert.X509Certificate;
import java.util.Iterator;

import javax.xml.crypto.AlgorithmMethod;
import javax.xml.crypto.KeySelector;
import javax.xml.crypto.KeySelectorException;
import javax.xml.crypto.KeySelectorResult;
import javax.xml.crypto.XMLCryptoContext;
import javax.xml.crypto.XMLStructure;
import javax.xml.crypto.dsig.keyinfo.KeyInfo;
import javax.xml.crypto.dsig.keyinfo.KeyName;
import javax.xml.crypto.dsig.keyinfo.KeyValue;
import javax.xml.crypto.dsig.keyinfo.PGPData;
import javax.xml.crypto.dsig.keyinfo.RetrievalMethod;
import javax.xml.crypto.dsig.keyinfo.X509Data;
import javax.xml.crypto.dsig.keyinfo.X509IssuerSerial;

import org.apache.log4j.Logger;

import com.sun.org.apache.xml.security.keys.content.MgmtData;
import com.sun.org.apache.xml.security.keys.content.SPKIData;

/**
 * Implements the {@link javax.xml.crypto.KeySelector KeySelector} interface for
 * the {@link javax.xml.crypto.dsig.dom.DOMValidateContext DOMValidateContext} 
 * in the method
 * {@link PEPClient#validateSignature(InputStream) PEPClient.validateSignature(InputStream)}.
 * Currently, this implementation supports only validation of 
 * {@link java.security.cert.X509Certificate X509Certificates} as
 * {@link javax.xml.crypto.dsig.keyinfo.KeyInfo KeyInfo Content objects}.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.6 $
 * @since
 */

public class PEPClientKeySelector extends KeySelector {
    static Logger logger = Logger.getLogger(PEPClientKeySelector.class);

    /**
     * Default constructor.
     */
    public PEPClientKeySelector() {
        super();
    }

    /**
     * Selects and validates the first
     * {@link java.security.cert.X509Certificate X509Certificates} from the list
     * of {@link javax.xml.crypto.dsig.keyinfo.KeyInfo KeyInfo Content objects}.
     * @return The public key from the certificate.
     * @throws KeySelectorException
     *             Signals any error during certificate validation.
     * @see javax.xml.crypto.KeySelector#select(javax.xml.crypto.dsig.keyinfo.KeyInfo,
     *      javax.xml.crypto.KeySelector.Purpose,
     *      javax.xml.crypto.AlgorithmMethod, javax.xml.crypto.XMLCryptoContext)
     */
    public KeySelectorResult select(KeyInfo keyInfo, Purpose purpose,
            AlgorithmMethod method, XMLCryptoContext context)
            throws KeySelectorException {
        KeySelectorResult ret = null;
        logger.debug("KEYSELECTOR START: ");
        logger.debug("KeyInfo.Id: " + keyInfo.getId());
        Iterator contentIt = keyInfo.getContent().iterator();
        while ((ret == null) && contentIt.hasNext()) {
            XMLStructure xmlStruct = (XMLStructure) contentIt.next();
            if (xmlStruct instanceof KeyValue) {
                PublicKey pk = null;
                try {
                    logger.debug("KeyValue: "
                            + ((KeyValue) xmlStruct).getPublicKey());
                    pk = ((KeyValue) xmlStruct).getPublicKey();
                } catch (KeyException e) {
                    throw new KeySelectorException(e.getCause());
                }
                //                ret = new PEPClientKeySelectorResult(pk);
            } else if (xmlStruct instanceof KeyName) {
                logger.debug("Keyname: " + ((KeyName) xmlStruct).getName());
            } else if (xmlStruct instanceof RetrievalMethod) {
                logger.debug("RetrievalMethod: URI="
                        + ((RetrievalMethod) xmlStruct).getURI());
            } else if (xmlStruct instanceof X509Data) {
                logger.debug("X509Data: ");
                Iterator X509DataIt = ((X509Data) xmlStruct).getContent()
                        .iterator();
                while ((ret == null) && X509DataIt.hasNext()) {
                    Object contentObj = X509DataIt.next();
                    if (contentObj instanceof String) {
                        logger.debug("X509Data.Content.SubjectName: "
                                + (String) contentObj);

                    } else if (contentObj instanceof byte[]) {
                        logger.debug("X509Data.Content.SubjectKeyId: ");
                    } else if (contentObj instanceof X509Certificate) {
                        X509Certificate cert = (X509Certificate) contentObj;
                        logger.debug("X509Data.Content.X509Certificate: "
                                + cert);
                        try {
                            validateCertificate(cert);
                            ret = new PEPClientKeySelectorResult(cert
                                    .getPublicKey());
                            logger.info("Certificate validation OK");
                        } catch (NoSuchAlgorithmException e) {
                            logger.error("NoSuchAlgorithmException :"
                                    + e.getMessage());
                            throw new KeySelectorException(e.getCause());
                        } catch (CertificateException e) {
                            logger.error("CertificateException: "
                                    + e.getMessage());
                            throw new KeySelectorException(e.getCause());
                        } catch (KeyStoreException e) {
                            logger
                                    .error("KeyStoreException :"
                                            + e.getMessage());
                            throw new KeySelectorException(e.getCause());
                        } catch (CertPathValidatorException e) {
                            logger.error("CertPathValidatorException: "
                                    + e.getMessage());
                            logger.error("CertPath= " + e.getCertPath()
                                    + " Index= " + e.getIndex());
                            throw new KeySelectorException(e.getCause());
                        } catch (InvalidAlgorithmParameterException e) {
                            logger.error("InvalidAlgorithmParameterException: "
                                    + e.getMessage());
                            throw new KeySelectorException(e.getCause());
                        } catch (CertPathBuilderException e) {
                            logger.error("CertPathBuilderException: "
                                    + e.getMessage());
                            throw new KeySelectorException(e.getCause());
                        } catch (IOException e) {
                            logger.error("IOException: " + e.getMessage());
                            throw new KeySelectorException(e.getCause());
                        }
                    } else if (contentObj instanceof X509CRL) {
                        logger.debug("X509Data.Content.X509CRL: "
                                + (X509CRL) contentObj);
                    } else if (contentObj instanceof X509IssuerSerial) {
                        X509IssuerSerial issuerSerial = (X509IssuerSerial) contentObj;
                        logger
                                .debug("X509Data.Content.X509IssuerSerial: IssuerMame= "
                                        + issuerSerial.getIssuerName()
                                        + " SerialNumber= "
                                        + issuerSerial.getSerialNumber());
                    } else {
                        logger.debug("X509Data.Content. UnsupportedDataType= "
                                + contentObj.getClass().getName());
                    }
                }
            } else if (xmlStruct instanceof PGPData) {
                logger.debug("PGPData:");
            } else if (xmlStruct instanceof SPKIData) {
                logger.debug("SPKIData:");
            } else if (xmlStruct instanceof MgmtData) {
                logger.debug("MgmtData:");
            } else if (xmlStruct instanceof RetrievalMethod) {
                logger.debug("RetrievalMethod: URI="
                        + ((RetrievalMethod) xmlStruct).getURI());
            } else {
                logger.warn("Unsupported XMLStructure: "
                        + xmlStruct.getClass().getName());
            }
        }

        if (purpose != null)
            logger.debug("Purpose: " + purpose.toString());
        else
            logger.debug("Purpose:null");
        if (method != null)
            logger.debug("AlgorithmMethod: Alg= "
                    + method.getAlgorithm()
                    + " AlgParamSpec= "
                    + (method.getParameterSpec() != null ? method
                            .getParameterSpec().toString() : "null"));
        else
            logger.debug("AlgorithmMethod:null");
        if (context != null)
            logger.debug("Context: BaseURI"
                    + context.getBaseURI()
                    + " KeySelector= "
                    + (context.getKeySelector() != null ? context
                            .getKeySelector().toString() : "null")
                    + " URIDereferencer= "
                    + (context.getURIDereferencer() != null ? context
                            .getURIDereferencer().toString() : "null"));
        else
            logger.debug("Context:null");
        logger.debug("KEYSELECTOR END: ");
        return ret;
    }

    /**
     * Validates an {@link java.security.cert.X509Certificate X509Certificate}.
     * The validation comprises a validity check and the certification path. The
     * validation takes places according to the default PKIX validation process.
     * It uses the default (SSL)trust store defined in the system properties
     * "javax.net.ssl.trustStore" and "javax.net.ssl.trustStorePassword".
     * 
     * @param cert
     *            The {@link java.security.cert.X509Certificate X509Certificate}
     *            to validate.
     * @throws NoSuchAlgorithmException
     * @throws CertificateException
     * @throws IOException
     * @throws KeyStoreException
     * @throws CertPathValidatorException
     * @throws InvalidAlgorithmParameterException
     * @throws CertPathBuilderException
     */
    private void validateCertificate(X509Certificate cert)
            throws NoSuchAlgorithmException, CertificateException, IOException,
            KeyStoreException, CertPathValidatorException,
            InvalidAlgorithmParameterException, CertPathBuilderException {
        cert.checkValidity();
        CertPathValidator cpValidator = CertPathValidator
                .getInstance(CertPathValidator.getDefaultType());
        logger.debug("CertPathValidator Algorithm="
                + cpValidator.getAlgorithm());
        CertPathBuilder cpBuilder = CertPathBuilder.getInstance(CertPathBuilder
                .getDefaultType());

        FileInputStream truststoreIn = new FileInputStream(System
                .getProperty("javax.net.ssl.trustStore"));
        KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
        trustStore.load(truststoreIn, System.getProperty(
                "javax.net.ssl.trustStorePassword").toCharArray());
        PKIXParameters cpParam = new PKIXParameters(trustStore);
        X509CertSelector targetConstraints = new X509CertSelector();
        targetConstraints.setCertificate(cert);
        PKIXBuilderParameters cpBuilderParam = new PKIXBuilderParameters(
                trustStore, targetConstraints);

        CertPathValidatorResult result = cpValidator.validate(cpBuilder.build(
                cpBuilderParam).getCertPath(), cpParam);

    }
}
