/*
 * $Id: LOPFilter.java,v 1.7 2006/07/24 17:31:09 suhrbier Exp $
 * Created on 07.08.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: LOPFilter.java,v $
 * Revision 1.7  2006/07/24 17:31:09  suhrbier
 * update JavaDoc
 *
 * Revision 1.6  2006/07/14 21:17:58  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.5  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.4  2005/08/15 13:24:46  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.3  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.2  2005/08/08 19:39:09  suhrbier
 * COP and LOPFilter tested.
 * ProtocolFilter enhancements.
 *
 * Revision 1.1  2005/08/07 16:15:32  suhrbier
 * Umstellung Request auf Filter
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import java.util.ArrayList;
import java.util.List;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.LOP;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * The class LOPFilter parses the content of a LOP operator of BioCASE search
 * request from a SAX stream.
 * <p>
 * All SAX events are forwarded to the defined ContentHandler. The parsed
 * {@link LOP LOP}may be retrieved with {@link #getOperator() getOperator()}.
 * </p>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.7 $
 * @since
 */

public class LOPFilter extends ProtocolFilter {

	/**
	 * Local name of the attribute <code>syntax</code> of any COP element
	 */
	protected static String LN_LOP_SYNTAX = "syntax";

	// Unary LOP
	/** Local name of the Unary LOP <code>not</code> */
	public static String LN_NOT = "not";

	// Binary LOP
	/** Local name of the Binary LOP <code>and</code> */
	public static String LN_AND = "and";

	/** Local name of the Binary LOP <code>or</code> */
	public static String LN_OR = "or";

	/** Stores the parsed LOP */
	protected LOP lop;

	/**
	 * Track mode for the filter stack
	 */
	static final int trackMode = FilterStack.TRACK_MODE_ATTRIBUTES
			| FilterStack.TRACK_MODE_LOCALNAME | FilterStack.TRACK_MODE_QNAME
			| FilterStack.TRACK_MODE_CHARACTERS;

	/** List of parsed suboperators (max 2) */
	private List opFilters;

	/**
	 * Inits the LOP filter.
	 * 
	 * @param protocolURI
	 *            URI of the protocol version.
	 * @param filterRoot
	 *            Local name of the root element.
	 * @param trackMode
	 *            Define track mode for the internal
	 *            {@link FilterStack FilterStack}.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
	 */
	public LOPFilter(String protocolURI, String filterRoot, int trackMode, int filterMode) {
		super(protocolURI, filterRoot, trackMode, filterMode);
		initSubFilter();
	}

	/**
	 * Inits the LOP filter with the specified parent.
	 * 
	 * @param parent
	 *            Parent reader.
	 * @param protocolURI
	 *            URI of the protocol version.
	 * @param filterRoot
	 *            Local name of the root element.
	 * @param trackMode
	 *            Define track mode for the internal
	 *            {@link FilterStack FilterStack}.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
	 */
	public LOPFilter(XMLReader parent, String protocolURI, String filterRoot,
			int trackMode, int filterMode) {
		super(parent, protocolURI, filterRoot, trackMode, filterMode);
		initSubFilter();
	}

	private void initSubFilter() {
	}

	/**
	 * Initialises the LOP setting its syntax.
	 * 
	 * @see ProtocolFilter#parseInit(SAXElement)
	 */
	protected SAXElement parseInit(SAXElement element)
			throws SAXException {
		lop = new LOP(getProtocolURI(), element.getLocalName());
		lop.setSyntax(element.getAttributeValue(LN_LOP_SYNTAX));
		return element;
	}

	/**
	 * Sets up the corresponding subfilter depending on the current operator type.
	 * The subfilter is added to the internal operator filter list and registered as subfilter.
	 * @see ProtocolFilter#filterElement(SAXElement)
	 */
	protected SAXElement filterElement(SAXElement element)
			throws SAXException {
		String operator = element.getLocalName();
		ProtocolFilter opFilter = null;
		// create operator filter depending on the element's type.
		if (COPFilter.isOperator(operator))
			opFilter = new COPFilter(getParent(), getProtocolURI(), operator, trackMode, getFilterMode());
		else if (LOPFilter.isOperator(operator))
			opFilter = new LOPFilter(getParent(), getProtocolURI(), operator, trackMode, getFilterMode());

		// for COP and LOP unary, set only one operator subfilter
		if (COPFilter.isOperator(operator) || LOPFilter.isUnary(operator)) {
			if (opFilters == null)
				opFilters = new ArrayList(1);
			if (opFilters.size() == 0)
				opFilters.add(opFilter);
			else
				opFilters.set(0, opFilter);
			// for LOP binary, add max 2 operator subfilters
		} else if (LOPFilter.isBinary(operator)) {
			if (opFilters == null)
				opFilters = new ArrayList(2);
			// already two opfilter in the list ?
			if (opFilters.size() < 2)
				// no, add this
				opFilters.add(opFilter);
			else
				// yes, replace the last
				opFilters.set(1, opFilter);
		}
		// register operator as subfilter
		registerSubFilter(operator, opFilter);
		return element;
	}

	/**
	 * Adds the last parsed operator to the lop.
	 * Unregisters the current operator subfilter.
	 * 
	 * @see ProtocolFilter#parseComplete(SAXElement)
	 */
	protected SAXElement parseElement(SAXElement element)
			throws SAXException {
		String operator = element.getLocalName();
		// Add the parsed operator (last in the list) to the lop.
		if (COPFilter.isOperator(operator))
			lop.addOperator(((COPFilter) opFilters.get(opFilters.size() - 1))
					.getOperator());
		else if (LOPFilter.isOperator(operator))
			lop.addOperator(((LOPFilter) opFilters.get(opFilters.size() - 1))
					.getOperator());
		// unregister current operator subfilter (the next must be determined within the next call to filterElement
		unregisterSubFilter(operator);
		return element;
	}

	/**
	 * Checks if the given operator is a LOP operator.
	 * 
	 * @param operator
	 *            operator to check
	 * @return <code>true</code> if operator is a LOP, <code>false</code>
	 *         otherwise.
	 */
	public static boolean isOperator(String operator) {
		return isUnary(operator) || isBinary(operator);
	}

	/**
	 * Checks if the given operator is a BinaryLOP operator.
	 * 
	 * @param operator
	 *            operator to check
	 * @return <code>true</code> if operator is binary, <code>false</code>
	 *         otherwise.
	 */
	public static boolean isBinary(String operator) {
		boolean ret = false;
		if (LN_AND.equals(operator) || LN_OR.equals(operator))
			ret = true;
		return ret;
	}

	/**
	 * Checks if the given operator is a UnaryLOP operator.
	 * 
	 * @param operator
	 *            operator to check
	 * @return <code>true</code> if operator is unary, <code>false</code>
	 *         otherwise.
	 */
	public static boolean isUnary(String operator) {
		boolean ret = false;
		if (LN_NOT.equals(operator))
			ret = true;
		return ret;
	}

	/**
	 * Gets the LOP operator
	 * 
	 * @return Returns the LOP operator.
	 */
	public LOP getOperator() {
		return lop;
	}

}
