/*
 * $Id: OperatorFilter.java,v 1.7 2006/07/14 21:17:57 suhrbier Exp $
 * Created on 07.08.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: OperatorFilter.java,v $
 * Revision 1.7  2006/07/14 21:17:57  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.6  2005/08/29 14:19:49  suhrbier
 * Fehlendes SetContentHandler korrigiert.
 *
 * Revision 1.5  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.4  2005/08/15 13:24:45  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.3  2005/08/10 20:57:12  suhrbier
 * Updated JavaDoc & Comments
 *
 * Revision 1.2  2005/08/08 19:39:09  suhrbier
 * COP and LOPFilter tested.
 * ProtocolFilter enhancements.
 *
 * Revision 1.1  2005/08/07 16:15:32  suhrbier
 * Umstellung Request auf Filter
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.Operator;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * The class OperatorFilter prepares the parsing of COP or LOP operators using
 * the classes {@link COPFilter COPFilter}or {@link LOPFilter LOPFilter}
 * respectively.
 * <p>
 * All SAX events are forwarded to the defined ContentHandler. The parsed
 * operator type may be retrieved with the method
 * {@link #getOperator() getOperator()}.
 * </p>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.7 $
 * @since
 */

public class OperatorFilter extends ProtocolFilter {

	/**
	 * Local name of the root element <code>filter</code>
	 */
	public static final String LN_ROOT = "filter";

	/**
	 * Track mode for the filter stack
	 */
	static final int trackMode = FilterStack.TRACK_MODE_ATTRIBUTES
			| FilterStack.TRACK_MODE_LOCALNAME | FilterStack.TRACK_MODE_QNAME
			| FilterStack.TRACK_MODE_CHARACTERS;

	/** Stores the parsed operator */
	Operator operator;

	/** Current subfilter parse COP or LOP */
	ProtocolFilter opFilter;

	/**
	 * Inits the filter instance with specified parent.
	 * 
	 * @param parent
	 *            parent reader.
	 * @param protocolURI
	 *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
	 */
	public OperatorFilter(XMLReader parent, String protocolURI, int filterMode) {
		super(parent, protocolURI, LN_ROOT, trackMode, filterMode);
	}

	/**
	 * Inits the filter instance.
	 * 
	 * @param protocolURI
	 *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
	 */
	public OperatorFilter(String protocolURI, int  filterMode) {
		super(protocolURI, LN_ROOT, trackMode, filterMode);
	}

	/**
	 * Inits the filter instance with specified parent.
	 * 
	 * @param parent
	 *            parent reader.
	 * @param protocolURI
	 *            protocol version URI of this filter.
     * @param rootElement
     *            Local name of the root element.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
	 */
	public OperatorFilter(XMLReader parent, String protocolURI,
			String rootElement, int filterMode) {
		super(parent, protocolURI, rootElement, trackMode, filterMode);
	}

	/**
	 * Inits the filter instance.
	 * 
	 * @param protocolURI
	 *            protocol version URI of this filter.
     * @param rootElement
     *            Local name of the root element.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
	 */
	public OperatorFilter(String protocolURI, String rootElement, int filterMode) {
		super(protocolURI, rootElement, trackMode, filterMode);
	}

	/**
	 * Gets the Operator
	 * 
	 * @return Returns the operator.
	 */
	public Operator getOperator() {
		return operator;
	}

	/**
	 * Registers COP or LOP as subfilter, depending on the first received
	 * element.
	 * 
	 * @see ProtocolFilter#filterElement(SAXElement)
	 */
	protected SAXElement filterElement(SAXElement element)
			throws SAXException {
		String operator = element.getLocalName();
		if (COPFilter.isOperator(operator))
			opFilter = new COPFilter(getParent(), getProtocolURI(), operator, trackMode, getFilterMode());
		else if (LOPFilter.isOperator(operator))
			opFilter = new LOPFilter(getParent(), getProtocolURI(), operator, trackMode, getFilterMode());
		opFilter.setContentHandler(getContentHandler());
		registerSubFilter(operator, opFilter);
		return element;
	}

	/**
	 * When subfilter finishes, stores the parsed operator.
	 * 
	 * @see ProtocolFilter#parseElement(SAXElement)
	 */
	protected SAXElement parseElement(SAXElement element)
			throws SAXException {
		String operator = element.getLocalName();
		if (COPFilter.isOperator(operator))
			this.operator = ((COPFilter) opFilter).getOperator();
		else if (LOPFilter.isOperator(operator))
			this.operator = ((LOPFilter) opFilter).getOperator();
		return element;
	}
}
