/*
 * $Id: ResponseFilter.java,v 1.18 2006/07/14 21:17:57 suhrbier Exp $
 * Created on 21.07.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: ResponseFilter.java,v $
 * Revision 1.18  2006/07/14 21:17:57  suhrbier
 * Basis fr Validation gelegt (Trennen von Signature und Document).
 *
 * Revision 1.17  2006/07/14 13:12:06  suhrbier
 * FilterMode integrieren
 *
 * Revision 1.16  2006/05/08 17:21:57  suhrbier
 * Transform Response into DOM Document.
 *
 * Revision 1.15  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.14  2005/09/02 19:46:00  suhrbier
 * + Denied treeElements are returned as Info-Diagnostics.
 *
 * Revision 1.13  2005/08/17 19:03:28  suhrbier
 * Durchgang wiederhergestellt, Rechteabfrage funktioniert nicht ?
 *
 * Revision 1.12  2005/08/15 21:58:39  suhrbier
 * Update of all Protocol Types finished.
 * Some tests are missing.
 *
 * Revision 1.11  2005/08/15 13:24:45  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.10  2005/08/08 19:39:09  suhrbier
 * COP and LOPFilter tested.
 * ProtocolFilter enhancements.
 *
 * Revision 1.9  2005/08/06 16:16:14  suhrbier
 * Zwischenspeicherung
 *
 * Revision 1.8  2005/08/05 14:28:03  suhrbier
 * Updated javadoc and comments.
 *
 * Revision 1.7  2005/08/04 23:28:49  suhrbier
 * Response komplett auf ProtocolFilter umgestellt.
 *
 * Revision 1.6  2005/07/25 18:59:00  suhrbier
 * Updated JavaDoc
 *
 * Revision 1.5  2005/07/25 15:30:07  suhrbier
 * HttpInputStream updated.
 *
 * Revision 1.4  2005/07/23 10:58:19  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/07/22 17:28:06  suhrbier
 * ResponseFilter implementiert, aber ungetestet.
 *
 * Revision 1.2  2005/07/21 21:18:38  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/07/21 19:00:32  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.biocase.protocol.filter;

import nbi.xml.sax.FilterStack;
import nbi.xml.sax.SAXElement;
import nbi.xmlsec.biocase.protocol.Diagnostics;
import nbi.xmlsec.biocase.protocol.Response;

import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * The ResponseFilter filters responses of the BioCASE protocol. It consists of
 * three internal filters working together:
 * <ul>
 * <li>HeaderFilter</li>
 * <li>RBACFilter</li>
 * <li>DiagnosticsFilter</li>
 * </ul>
 * 
 * The HeaderFilter simply reads in the BioCASE header and forwards all incoming
 * SAX Events to the ContentHandler. The RBACFilter cooperates with an RBACPDP
 * and possibly filters out treeElements according to the PDP's policies. The
 * result of this filtering process may be added to the Diagnostics of the
 * response, which is the task of the Diagnostics Filter.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.18 $
 * @since
 */

public class ResponseFilter extends ProtocolFilter {

    /** Local name of the element <code>response</code> */
    public static final String LN_ROOT = "response";

    /** Track mode for the filter stack */
    final static int trackMode = FilterStack.TRACK_MODE_ATTRIBUTES
            | FilterStack.TRACK_MODE_LOCALNAME | FilterStack.TRACK_MODE_QNAME;

    /** Subfilter for the header */
    HeaderFilter headerFilter = null;

    /** Subfilter for the content */
    ContentFilter contentFilter = null;

    /** Subfilter for the diagnostics */
    DiagnosticsFilter diagnosticsFilter = null;

    /** The parsed response */
    Response response;

    Diagnostics contentDiagnostics;

    /**
     * Inits the filter instance.
     * 
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public ResponseFilter(String protocolURI, int filterMode) {
        super(protocolURI, LN_ROOT, trackMode, filterMode);
    }

    /**
     * Inits the filter instance with specified parent.
     * 
     * @param parent
     *            parent reader.
     * @param protocolURI
     *            protocol version URI of this filter.
     * @param filterMode
     *            filterMode of this filter
     *            {@link ProtocolFilter#filterMode ProtocolFilter.filterMode}.
     */
    public ResponseFilter(XMLReader parent, String protocolURI, int filterMode) {
        super(parent, protocolURI, LN_ROOT, trackMode, filterMode);
    }

    /**
     * Registers the subfilters for header, content and Diagnostics.
     * 
     * @see ProtocolFilter#parseInit(SAXElement)
     */
    protected SAXElement parseInit(SAXElement element) throws SAXException {
        headerFilter = new HeaderFilter(getParent(), getProtocolURI(),
                getFilterMode());
        headerFilter.setContentHandler(getContentHandler());
        registerSubFilter(HeaderFilter.LN_ROOT, headerFilter);
        registerSubFilter(ContentFilter.LN_ROOT, null);
        diagnosticsFilter = new DiagnosticsFilter(getParent(),
                getProtocolURI(), getFilterMode());
        diagnosticsFilter.setContentHandler(getContentHandler());
        registerSubFilter(DiagnosticsFilter.LN_ROOT, diagnosticsFilter);
        return element;
    }

    /**
     * Sets the parameters of the content filter (especially the response
     * header) and overwrites the current registration before the content filter
     * is active. This is necessary, because the response header is unknown,
     * until parsing of the response header is complete.
     * 
     * @see ProtocolFilter#parseElement(SAXElement)
     */
    protected SAXElement filterElement(SAXElement element) throws SAXException {
        if (element.getLocalName().equals(ContentFilter.LN_ROOT)) {
            contentFilter = new ContentFilter(getParent(), element.getURI()
                    .toString(), getFilterMode());
            contentFilter.setContentHandler(getContentHandler());
            this.contentDiagnostics = new Diagnostics(getProtocolURI());
            if (getFilterMode() == ProtocolFilter.FILTERMODE_FILTER)
                contentFilter.setFilterParameter(new ContentFilterParameter(
                        getParameter().getRequest(), headerFilter.getHeader(),
                        getParameter().getPdp(), getParameter().getRoles(),
                        this.contentDiagnostics));
            registerSubFilter(ContentFilter.LN_ROOT, contentFilter);
        } else if (element.getLocalName().equals(DiagnosticsFilter.LN_ROOT)) {
            diagnosticsFilter.addDiagnostics(this.contentDiagnostics);
        }

        return element;
    }

    /**
     * Collects the response from the filters.
     * 
     * @see ProtocolFilter#parseComplete(SAXElement)
     */
    protected SAXElement parseComplete(SAXElement element) throws SAXException {
        response = new Response(getProtocolURI());
        response.setHeader(headerFilter.getHeader());
        response.setContent(contentFilter.getContent());
        diagnosticsFilter.getDiagnostics().addDiagnostics(
                this.contentDiagnostics);
        response.setDiagnostics(diagnosticsFilter.getDiagnostics());
        return element;
    }

    /**
     * Gets the parameter of this filter.
     * 
     * @return filter parameter.
     */
    protected ResponseFilterParameter getParameter() {
        return (ResponseFilterParameter) getFilterParameter();
    }

    /**
     * Gets the contentFilter
     * 
     * @return Returns the contentFilter.
     */
    public ContentFilter getContentFilter() {
        return contentFilter;
    }

    /**
     * Sets contentFilter
     * 
     * @param contentFilter
     *            The contentFilter to set.
     */
    public void setContentFilter(ContentFilter contentFilter) {
        this.contentFilter = contentFilter;
    }

    /**
     * Gets the diagnosticsFilter
     * 
     * @return Returns the diagnosticsFilter.
     */
    public DiagnosticsFilter getDiagnosticsFilter() {
        return diagnosticsFilter;
    }

    /**
     * Sets diagnosticsFilter
     * 
     * @param diagnosticsFilter
     *            The diagnosticsFilter to set.
     */
    public void setDiagnosticsFilter(DiagnosticsFilter diagnosticsFilter) {
        this.diagnosticsFilter = diagnosticsFilter;
    }

    /**
     * Gets the headerFilter
     * 
     * @return Returns the headerFilter.
     */
    public HeaderFilter getHeaderFilter() {
        return headerFilter;
    }

    /**
     * Sets headerFilter
     * 
     * @param headerFilter
     *            The headerFilter to set.
     */
    public void setHeaderFilter(HeaderFilter headerFilter) {
        this.headerFilter = headerFilter;
    }

    /**
     * Gets the response
     * 
     * @return Returns the response.
     */
    public Response getResponse() {
        return response;
    }
}
