/*
 * $Id: AbstractPolicyBuilder.java,v 1.4 2005/10/14 18:31:00 suhrbier Exp $
 * Created on 07.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: AbstractPolicyBuilder.java,v $
 * Revision 1.4  2005/10/14 18:31:00  suhrbier
 * *** empty log message ***
 *
 * Revision 1.3  2005/10/10 11:20:27  suhrbier
 * +Apply(Type)Builder
 * +CollectionUtils + Tests
 *
 * Revision 1.2  2005/10/07 18:27:45  suhrbier
 * Beginn Umbau XACML-Builder (4all PolicyTreeElements).
 *
 * Revision 1.1  2005/10/07 17:05:51  suhrbier
 * Beginn Umbau XACML-Builder.
 *
 */
package nbi.xmlsec.xacml.builder;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import nbi.util.CollectionUtil;

import org.apache.log4j.Logger;

import com.sun.xacml.Obligation;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.UnknownIdentifierException;
import com.sun.xacml.combine.CombiningAlgFactory;
import com.sun.xacml.combine.CombiningAlgorithm;

/**
 * Abstract base class for all policy building classes.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.4 $
 * @since
 */

abstract public class AbstractPolicyBuilder extends
        AbstractPolicyTreeElementBuilder {

    static Logger logger = Logger.getLogger(AbstractPolicyBuilder.class);

    /**
     * the XPath version to use for selectors
     */
    protected String defaultVersion = null;

    /**
     * The combining algorithm to use
     */
    protected CombiningAlgorithm combiningAlg = null;

    /** The policy's subelements */
    protected List treeElements = null;

    /** The policy's obligations */
    protected Set obligations = null;

    /**
     * Default constructor.
     */
    protected AbstractPolicyBuilder() {
        super();
    }

    /**
     * Sets the policy's combining algorithm.
     * 
     * @param combiningAlg
     *            The combining algorithm to set.
     */
    public void setCombiningAlg(CombiningAlgorithm combiningAlg) {
        this.combiningAlg = combiningAlg;
    }

    /**
     * Sets the policy's combining algorithm.
     * 
     * @param combiningAlgId
     *            The identifier of the combining algorithm to set.
     * @throws UnknownIdentifierException
     * @see CombiningAlgFactory#createAlgorithm(java.net.URI).
     */
    public void setCombiningAlg(URI combiningAlgId)
            throws UnknownIdentifierException {
        CombiningAlgFactory combiningAlgFactory = CombiningAlgFactory
                .getInstance();
        this.combiningAlg = combiningAlgFactory.createAlgorithm(combiningAlgId);
    }

    /**
     * Sets the policy's combining algorithm.
     * 
     * @param combiningAlgId
     *            The identifier of the combining algorithm to set.
     * @throws UnknownIdentifierException
     * @see CombiningAlgFactory#createAlgorithm(java.net.URI).
     */
    public void setCombiningAlg(String combiningAlgId)
            throws UnknownIdentifierException {
        setCombiningAlg(URI.create(combiningAlgId));
    }

    /**
     * Sets the XPath version to use for selectors
     * 
     * @param defaultVersion
     *            the XPath version to use for selectors.
     */
    public void setDefaultVersion(String defaultVersion) {
        this.defaultVersion = defaultVersion;
    }

    /**
     * Adds an obligation to the policy's obligations.
     * 
     * @param obligation
     *            The policy's obligations to set.
     * @return true if the obligation has been added succesfully.
     */
    public boolean add(Obligation obligation) {
        try {
            obligations = (Set) CollectionUtil.create(obligations,
                    HashSet.class);
        } catch (InstantiationException e) {
            logger.fatal("The exception " + e + "should never happen!");
        } catch (IllegalAccessException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        return CollectionUtil.add(obligations, obligation);
    }

    /**
     * Adds an obligation to the policy's obligation set.
     * 
     * @param obligation
     *            The policy's obligation to set.
     * @return true if the obligation has been added succesfully.
     */
    public boolean add(ObligationBuilder obligation) {
        return add(obligation.getInstance());
    }

    /**
     * Removes an obligation from the policy's obligation set.
     * 
     * @param obligation
     *            The policy's obligation to remove.
     * @return true if the obligation has been removed succesfully.
     */
    public boolean remove(Obligation obligation) {
        return CollectionUtil.remove(obligations,obligation);
    }

    /**
     * Sets obligations
     * 
     * @param obligations
     *            The obligations to set.
     */
    public void setObligations(Set obligations) {
        this.obligations = obligations;
    }

    /**
     * Adds a tree element to this policy.
     * 
     * @param element
     *            The tree element to add.
     * @return true if the tree element has been added succesfully.
     */
    protected boolean add(PolicyTreeElement element) {
        try {
            treeElements = (List) CollectionUtil.create(treeElements,
                    ArrayList.class);
        } catch (InstantiationException e) {
            logger.fatal("The exception " + e + "should never happen!");
        } catch (IllegalAccessException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        return CollectionUtil.add(treeElements, element);
    }

    /**
     * Adds a tree element to this policy.
     * 
     * @param element
     *            The tree element to add.
     * @return true if the tree element has been added succesfully.
     */
    protected boolean add(AbstractPolicyTreeElementBuilder element) {
        return treeElements.add(element.getInstance());
    }

    /**
     * Removes a tree element from this policy.
     * 
     * @param element
     *            The tree element to remove.
     * @return true if the tree element has been removed succesfully.
     */
    protected boolean remove(PolicyTreeElement element) {
        return CollectionUtil.remove(treeElements,element);
    }

    /**
     * Sets the treeElements of this policy.
     * 
     * @param elements
     *            The policies to set.
     */
    protected void setTreeElements(List elements) {
        this.treeElements = elements;
    }

}
