/*
 * $Id: RequestCtxBuilder.java,v 1.3 2006/04/24 19:50:20 suhrbier Exp $
 * Created on 19.04.2006
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RequestCtxBuilder.java,v $
 * Revision 1.3  2006/04/24 19:50:20  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2006/04/20 17:15:27  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2006/04/19 09:55:43  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import java.util.HashSet;
import java.util.Set;

import nbi.util.CollectionUtil;

import org.apache.log4j.Logger;

import com.sun.xacml.ctx.Attribute;
import com.sun.xacml.ctx.RequestCtx;
import com.sun.xacml.ctx.Subject;

/**
 * Class to create a RequestCtx.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.3 $
 * @since
 */

public class RequestCtxBuilder {
    static Logger logger = Logger.getLogger(RequestCtxBuilder.class);

    /** Attribute type resource */
    protected final int AT_RESOURCE = 1;

    /** Attribute type action */
    protected final int AT_ACTION = 2;

    /** Attribute type environment */
    protected final int AT_ENV = 3;

    /** Set of subject attributes */
    private Set subjects = new HashSet();

    /** Set of resource attributes */
    private Set resources = new HashSet();

    /** Set of action attribute */
    private Set actions = new HashSet();

    /** Set of environment attributes */
    private Set environment = new HashSet();

    /**
     * 
     */
    public RequestCtxBuilder() {
        super();
    }

    /**
     * Adds a subject to the {@link RequestCtx RequestCtx}'s subjects.
     * 
     * @param subject
     *            The subject to add.
     * @return true if the subject has been added succesfully.
     */
    public boolean add(Subject subject) {
        try {
            subjects = (Set) CollectionUtil.create(subjects, HashSet.class);
        } catch (InstantiationException e) {
            logger.fatal("The exception " + e + "should never happen!");
        } catch (IllegalAccessException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        return CollectionUtil.add(subjects, subject);
    }

    /**
     * Removes a subject from the {@link RequestCtx RequestCtx}'s subjects.
     * 
     * @param subject
     *            The RequestCtx's subject to remove.
     * @return true if the subject has been removed succesfully.
     */
    public boolean remove(Subject subject) {
        return CollectionUtil.remove(subjects, subject);
    }

    /**
     * Adds an attribute of the given type to the related
     * {@link RequestCtx RequestCtx}'s attribute set. The supported attribute
     * types are:
     * <li>{@link #AT_ACTION AT_ACTION}</li>
     * <li>{@link #AT_RESOURCE AT_RESOURCE}</li>
     * <li>{@link #AT_ENV AT_ENV}</li>
     * 
     * @param type
     *            Type of attribute set.
     * @param attr
     *            The attribute to add.
     * @return true if the attribute has been added succesfully.
     */
    protected boolean add(int type, Attribute attr) {
        boolean ret = false;
        try {
            switch (type) {
            case AT_RESOURCE:
                resources = (Set) CollectionUtil.create(resources,
                        HashSet.class);
                break;
            case AT_ACTION:
                actions = (Set) CollectionUtil.create(actions, HashSet.class);
                break;
            case AT_ENV:
                environment = (Set) CollectionUtil.create(environment,
                        HashSet.class);
                break;
            default:
                throw new IllegalArgumentException(
                        "Parameter is not an RequestCtx attribute");
            }
        } catch (InstantiationException e) {
            logger.fatal("The exception " + e + "should never happen!");
        } catch (IllegalAccessException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        switch (type) {
        case AT_RESOURCE:
            ret = CollectionUtil.add(resources, attr);
            break;
        case AT_ACTION:
            ret = CollectionUtil.add(actions, attr);
            break;
        case AT_ENV:
            ret = CollectionUtil.add(environment, attr);
            break;
        }
        return ret;
    }

    /**
     * Removes an attribute of the given type from the related
     * {@link RequestCtx RequestCtx}'s attribute set. The supported attribute
     * types are:
     * <li>{@link #AT_ACTION AT_ACTION}</li>
     * <li>{@link #AT_RESOURCE AT_RESOURCE}</li>
     * <li>{@link #AT_ENV AT_ENV}</li>
     * 
     * @param type
     *            Type of attribute set.
     * @param attr
     *            The RequestCtx's attr to remove.
     * @return true if the attribute has been removed succesfully.
     */
    protected boolean remove(int type, Attribute attr) {
        boolean ret = false;
        switch (type) {
        case AT_RESOURCE:
            ret = CollectionUtil.remove(resources, attr);
            break;
        case AT_ACTION:
            ret = CollectionUtil.remove(actions, attr);
            break;
        case AT_ENV:
            ret = CollectionUtil.remove(environment, attr);
            break;
        default:
            throw new IllegalArgumentException(
                    "Parameter is not an RequestCtx attribute");
        }
        return ret;
    }

    /**
     * Adds a resource attribute to the {@link RequestCtx RequestCtx}'s
     * resources.
     * 
     * @param attr
     *            The attribute to add.
     * @return true if the attribute has been added succesfully.
     */
    public boolean addResource(Attribute attr) {
        return add(AT_RESOURCE, attr);
    }

    /**
     * Removes a resource attribute from the {@link RequestCtx RequestCtx}'s
     * resources.
     * 
     * @param attr
     *            The attribute to add.
     * @return true if the attribute has been added succesfully.
     */
    public boolean removeResource(Attribute attr) {
        return remove(AT_RESOURCE, attr);
    }

    /**
     * Adds an action attribute to the {@link RequestCtx RequestCtx}'s
     * resources.
     * 
     * @param attr
     *            The attribute to add.
     * @return true if the attribute has been added succesfully.
     */
    public boolean addAction(Attribute attr) {
        return add(AT_ACTION, attr);
    }

    /**
     * Removes an action attribute from the {@link RequestCtx RequestCtx}'s
     * resources.
     * 
     * @param attr
     *            The attribute to add.
     * @return true if the attribute has been added succesfully.
     */
    public boolean removeAction(Attribute attr) {
        return remove(AT_ACTION, attr);
    }

    /**
     * Adds an environemnt attribute to the {@link RequestCtx RequestCtx}'s
     * resources.
     * 
     * @param attr
     *            The attribute to add.
     * @return true if the attribute has been added succesfully.
     */
    public boolean addEnvironment(Attribute attr) {
        return add(AT_ENV, attr);
    }

    /**
     * Removes an environment attribute from the {@link RequestCtx RequestCtx}
     * 's resources.
     * 
     * @param attr
     *            The attribute to add.
     * @return true if the attribute has been added succesfully.
     */
    public boolean removeEnvironment(Attribute attr) {
        return remove(AT_ENV, attr);
    }

    /**
     * Creates a new {@link RequestCtx RequestCtx}.
     * 
     * @return The new RequestCtx or null <
     */
    public RequestCtx getInstance() {
        return new RequestCtx(subjects, resources, actions, environment);
    }
}
