/*
 * $Id: TargetMatchBuilderFactory.java,v 1.4 2005/11/04 17:52:36 suhrbier Exp $
 * Created on 15.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: TargetMatchBuilderFactory.java,v $
 * Revision 1.4  2005/11/04 17:52:36  suhrbier
 * Permission add mit Targets funzt.
 *
 * Revision 1.3  2005/10/21 17:43:03  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2005/10/15 21:37:39  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/15 18:00:51  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.builder;

import java.util.HashMap;

import nbi.util.MapUtil;

import org.apache.log4j.Logger;

import com.sun.xacml.TargetMatch;

/**
 * Implements a factory for TargetMatchBuilders. TargetMatchBuilders have to be
 * registered under the key of the related object class. The
 * {@link #getInstance(int, Object) getInstance(int, Object)}method returns the
 * matching {@link TargetMatchBuilder TargetMatchBuilder}instance, initialised
 * with the given object as attribute.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.4 $
 * @since
 */

public class TargetMatchBuilderFactory {
    static Logger logger = Logger.getLogger(TargetMatchBuilderFactory.class);

    /** Registry for subject match builders */
    HashMap subjectMatchBuilder;

    /** Registry for resource match builders */
    HashMap resourceMatchBuilder;

    /** Registry for action match builders */
    HashMap actionMatchBuilder;

    /**
     * Creates an instance of a matching
     * {@link TargetMatchBuilder TargetMatchBuilder}of the given object.
     * 
     * @param type
     *            {@link TargetMatchBuilder TargetMatchBuilder}type (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * 
     * @param obj
     *            Object to create a matching
     *            {@link TargetMatchBuilder TargetMatchBuilder}for.
     * @return {@link TargetMatchBuilder TargetMatchBuilder}instance or
     *         <code>null</code>.
     */
    public TargetMatchBuilder getInstance(int type, Object obj) {
        TargetMatchBuilder result = null;
        switch (type) {
        case TargetMatch.SUBJECT:
            result = (TargetMatchBuilder) subjectMatchBuilder.get(obj
                    .getClass());
            break;
        case TargetMatch.RESOURCE:
            result = (TargetMatchBuilder) resourceMatchBuilder.get(obj
                    .getClass());
            break;
        case TargetMatch.ACTION:
            result = (TargetMatchBuilder) actionMatchBuilder
                    .get(obj.getClass());
            break;
        }
        if (result != null) {
            result.setAttribute(obj);
        }
        return result;
    }

    /**
     * Registers a {@link SubjectMatchBuilder SubjectMatchBuilder}for a given
     * object class.
     * 
     * @param objClass
     *            Object class to register.
     * @param builder
     *            {@link SubjectMatchBuilder SubjectMatchBuilder}related to the
     *            object class.
     */
    public void registerSubjectMatchBuilder(Class objClass,
            TargetMatchBuilder builder) {
        registerTargetMatchBuilder(TargetMatch.SUBJECT, objClass, builder);
    }

    /**
     * Registers a {@link SubjectMatchBuilder SubjectMatchBuilder}for a given
     * object class.
     * 
     * @param objClass
     *            Object class to register.
     * @param builder
     *            {@link ResourceMatchBuilder ResourceMatchBuilder}related to
     *            the object class.
     */
    public void registerResourceMatchBuilder(Class objClass,
            TargetMatchBuilder builder) {
        registerTargetMatchBuilder(TargetMatch.RESOURCE, objClass, builder);
    }

    /**
     * Registers a {@link ActionMatchBuilder ActionMatchBuilder}for a given
     * object class.
     * 
     * @param objClass
     *            Object class to register.
     * @param builder
     *            {@link ActionMatchBuilder ActionMatchBuilder}related to the
     *            object class.
     */
    public void registerActionMatchBuilder(Class objClass,
            TargetMatchBuilder builder) {
        registerTargetMatchBuilder(TargetMatch.ACTION, objClass, builder);
    }

    /**
     * Registers a {@link TargetMatchBuilder TargetMatchBuilder}for a given
     * object class.
     * 
     * @param type
     *            {@link TargetMatchBuilder TargetMatchBuilder}type (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @param objClass
     *            Object class to register.
     * @param builder
     *            {@link TargetMatchBuilder TargetMatchBuilder}related to the
     *            object class.
     */
    protected void registerTargetMatchBuilder(int type, Class objClass,
            TargetMatchBuilder builder) {
        try {
            switch (type) {
            case TargetMatch.SUBJECT:
                subjectMatchBuilder = (HashMap) MapUtil.create(
                        subjectMatchBuilder, HashMap.class);
                MapUtil.put(subjectMatchBuilder, objClass, builder);
                break;
            case TargetMatch.RESOURCE:
                resourceMatchBuilder = (HashMap) MapUtil.create(
                        resourceMatchBuilder, HashMap.class);
                MapUtil.put(resourceMatchBuilder, objClass, builder);
                break;
            case TargetMatch.ACTION:
                actionMatchBuilder = (HashMap) MapUtil.create(
                        actionMatchBuilder, HashMap.class);
                MapUtil.put(actionMatchBuilder, objClass, builder);
                break;
            }
        } catch (InstantiationException e) {
            logger.fatal("The exception " + e + "should never happen!");
        } catch (IllegalAccessException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
    }

    /**
     * Unregisters a {@link SubjectMatchBuilder SubjectMatchBuilder}for a given
     * object class.
     * 
     * @param objClass
     *            Object class to unregister.
     */
    public void unregisterSubjectMatchBuilder(Class objClass) {
        unregisterTargetMatchBuilder(TargetMatch.SUBJECT, objClass);
    }

    /**
     * Unregisters a {@link SubjectMatchBuilder SubjectMatchBuilder}for a given
     * object class.
     * 
     * @param objClass
     *            Object class to register.
     */
    public void unregisterResourceMatchBuilder(Class objClass) {
        unregisterTargetMatchBuilder(TargetMatch.RESOURCE, objClass);
    }

    /**
     * Unregisters a {@link ActionMatchBuilder ActionMatchBuilder}for a given
     * object class.
     * 
     * @param objClass
     *            Object class to register.
     */
    public void unregisterActionMatchBuilder(Class objClass) {
        unregisterTargetMatchBuilder(TargetMatch.ACTION, objClass);
    }

    /**
     * Unregisters a {@link TargetMatchBuilder TargetMatchBuilder}of an object
     * class.
     * 
     * @param type
     *            {@link TargetMatchBuilder TargetMatchBuilder}type (
     *            {@link TargetMatch#SUBJECT SUBJECT},
     *            {@link TargetMatch#RESOURCE RESOURCE}or
     *            {@link TargetMatch#ACTION ACTION}).
     * @param objClass
     *            Object class to unregister.
     */
    protected void unregisterTargetMatchBuilder(int type, Class objClass) {
        switch (type) {
        case TargetMatch.SUBJECT:
            MapUtil.remove(subjectMatchBuilder, objClass);
            break;
        case TargetMatch.RESOURCE:
            MapUtil.remove(resourceMatchBuilder, objClass);
            break;
        case TargetMatch.ACTION:
            MapUtil.remove(actionMatchBuilder, objClass);
            break;
        }
    }
}
