/*
 * $Id:$
 * Created on 08.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log:$
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.net.URI;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.security.auth.x500.X500Principal;

import nbi.xmlsec.xacml.builder.PermitRuleBuilder;
import nbi.xmlsec.xacml.profile.rbac.builder.AssignmentTargetMatchBuilderFactory;
import nbi.xmlsec.xacml.profile.rbac.parser.AssignmentTargetMatchParserFactory;

import org.apache.log4j.Logger;

import com.sun.xacml.ParsingException;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Rule;
import com.sun.xacml.TargetMatch;
import com.sun.xacml.ctx.Result;

/**
 * This class implements a role assignment rule according to the XACML RBAC
 * Profile. It assigns subjects to a role. Currently, only subjects with X500
 * names are supported. The assignment may be represented as a {@link Rule Rule}
 * or parsed from a {@link Rule Rule}representation.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.14 $
 * @since
 */

public class Assignment extends RBACPolicyElement {
    static Logger logger = Logger.getLogger(Assignment.class);

    /**
     * This identifier MAY be used as the <AttributeValue>of the
     * &action;action-identifier Attribute in a RolePolicySet Assignment <Policy>.
     */
    public final static URI enableRoleActionId = URI
            .create("urn:oasis:names:tc:xacml:2.0:actions:enableRole");

    /**
     * Create an Assigment instance.
     * 
     * @param domain
     *            Domain of the assignment identifier.
     * @param label
     *            Label of the assignment identifier.
     * @param roleId
     *            RolePolicySet for this assignment.
     */
    public Assignment(String domain, String label, RoleValueIdentifier roleId) {
        setIdentifier(new AssignmentIdentifier(domain, label));
        setTarget(new RBACTarget(AssignmentTargetMatchBuilderFactory
                .getInstance(), AssignmentTargetMatchParserFactory
                .getInstance()));
        setRoleIdentifier(roleId);
        setEnableRoleActionId();
    }

    /**
     * Create an Assigment instance.
     * 
     * @param assignmentId
     *            Assignment identifier (domain:assignment:label).
     * @param roleId
     *            RolePolicySet for this assignment.
     */
    public Assignment(URI assignmentId, RoleValueIdentifier roleId)
            throws ParsingException {
        setIdentifier(assignmentId);
        setTarget(new RBACTarget(AssignmentTargetMatchBuilderFactory
                .getInstance(), AssignmentTargetMatchParserFactory
                .getInstance()));
        setRoleIdentifier(roleId);
        setEnableRoleActionId();
    }

    /**
     * Create an Assigment instance from its {@link Rule Rule}representation.
     * 
     * @param rule
     *            {@link Rule Rule}representation.
     */
    public Assignment(Rule rule) throws ParsingException {
        setTarget(new RBACTarget(AssignmentTargetMatchBuilderFactory
                .getInstance(), AssignmentTargetMatchParserFactory
                .getInstance()));
        parse(rule);
    }

    /**
     * Adds a set of subjects to this assigment.
     * 
     * @param subjects
     *            The subjects to add.
     * @return <code>true</code> if the subjects has been added succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean add(Set subjects) {
        return getTarget().addSubject(subjects);
    }

    /**
     * Removes a set of subjects from this assigment.
     * 
     * @param subjects
     *            The subjects to add.
     * @return <code>true</code> if the subjects has been added succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean remove(Set subjects) {
        return getTarget().removeSubject(subjects);
    }

    /**
     * Adds an X500Principal subject to the subject set.
     * 
     * @param subject
     *            The subject to add.
     * @return <code>true</code> if the subject has been added succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean add(X500Principal subject) {
        return getTarget().addSubject(subject);
    }

    /**
     * Removes an X500Principal subject to the subject set.
     * 
     * @param subject
     *            The subject to remove.
     * @return <code>true</code> if the subject has been removed succesfully,
     *         <code>false</code> otherwise.
     */
    public boolean remove(X500Principal subject) {
        return getTarget().removeSubject(subject);
    }

    /**
     * Adds the subjects of another Assignment to the subjects of this assignment.
     * @param element The assignment to add.
     * @return <code>true</code> if the assignment has been added succesfully,
     *         <code>false</code> otherwise.
     * @see RBACPolicyElement#add(RBACPolicyElement)
     */
    public boolean add(RBACPolicyElement element) {
        boolean result=false;
        if(element instanceof Assignment)
        {
            result=add(((Assignment)element).getSubjects());
        }
        return result;
    }
    /**
     * Removes the subjects of another Assignment from the subjects of this assignment.
     * @param element The assignment to remove.
     * @return <code>true</code> if the assignment has been removed succesfully,
     *         <code>false</code> otherwise.
     * @see RBACPolicyElement#remove(RBACPolicyElement)
     */
    public boolean remove(RBACPolicyElement element) {
        boolean result=false;
        if(element instanceof Assignment)
        {
            result=remove(((Assignment)element).getSubjects());
        }
        return result;
    }
    /**
     * Returns the {@link Rule Rule}representation of the assignment.
     * 
     * @return Assignment's rule representation.
     */
    public PolicyTreeElement toPolicyTreeElement() {
        PermitRuleBuilder ruleBuilder = new PermitRuleBuilder(getIdentifier()
                .toURI());
        ruleBuilder.setTarget(getTarget().toTarget());
        return (Rule) ruleBuilder.getInstance();
    }

    /**
     * Parses an assignment from its {@link Rule Rule}representation.
     * 
     * @param element
     *            The rule to parse.
     * @throws ParsingException
     *             Indicates a malformed {@link Rule Rule}representation.
     */
    public void parse(PolicyTreeElement element) throws ParsingException {
        if (element == null)
            throw new ParsingException("Tree element is null");
        
        Rule rule = null;
        if (element instanceof Rule)
            rule = (Rule) element;
        setIdentifier(rule.getId());
        if (rule.getEffect() != Result.DECISION_PERMIT)
            throw new ParsingException(
                    "Assignment rules must evaluate to permit.");
        getTarget().parse(rule.getTarget());

        if (!enableRoleActionId.equals(getEnableRoleActionId()))
            throw new ParsingException("Not an assignment match value."
                    + getEnableRoleActionId());
    }

    private void parseTargetResources(List resources) throws ParsingException {
        if ((resources != null) || (resources.size() != 1))
            throw new ParsingException(
                    "Assignments must have only one resource section.");
        List resourceMatches = (List) resources.get(0);
        if ((resourceMatches != null) || (resourceMatches.size() != 1))
            throw new ParsingException(
                    "Assignments must have only one resource match.");
        TargetMatch resourceMatch = (TargetMatch) resourceMatches.get(0);
        if (resourceMatch.getType() != TargetMatch.RESOURCE)
            throw new ParsingException("Wrong TargetMatch type."
                    + resourceMatch.getType());
    }

    private void checkTargetActions(List actions) throws ParsingException {
        if ((actions != null) || (actions.size() != 1))
            throw new ParsingException(
                    "Assignments must have only one action section.");
        List actionMatches = (List) actions.get(0);
        if ((actionMatches != null) || (actionMatches.size() != 1))
            throw new ParsingException(
                    "Assignments must have only one action match.");
        TargetMatch actionMatch = (TargetMatch) actionMatches.get(0);
        if (actionMatch.getType() != TargetMatch.ACTION)
            throw new ParsingException("Wrong TargetMatch type."
                    + actionMatch.getType());
    }

    /**
     * Gets the EnableRoelActionId from the target.
     * 
     * @return The assignment's role identifier.
     */
    public URI getEnableRoleActionId() {
        return (URI) getTarget().actionIterator().next();
    }

    /**
     * Sets the EnableRoelActionId on the target.
     * 
     * @return The assignment's role identifier.
     */
    protected void setEnableRoleActionId() {
        getTarget().clearActions();
        getTarget().addAction(enableRoleActionId);
    }

    /**
     * Gets the role identifier of this assignment.
     * 
     * @return The assignment's role identifier.
     */
    public RoleValueIdentifier getRoleIdentifier() {
        return (RoleValueIdentifier) getTarget().resourceIterator().next();
    }

    /**
     * Sets the role identifier of this assignment.
     * 
     * @param roleId
     *            The assignment's role identifier.
     */
    public void setRoleIdentifier(RoleValueIdentifier roleId) {
        getTarget().clearResources();
        getTarget().addResource(roleId);
    }

    /**
     * Sets the role identifier of this assignment.
     * 
     * @param roleId
     *            The assignment's role identifier.
     * @throws ParsingException
     *             Indicates a malformed
     *             {@link RoleValueIdentifier RoleValueIdentifier}
     *             representation.
     */
    public void setRoleIdentifier(URI roleId) throws ParsingException {
        getTarget().clearResources();
        getTarget().addResource(
                (RoleValueIdentifier) Identifier.getInstance(roleId));
    }

    /**
     * Gets the assigned subjects of this this assignment.
     * 
     * @return The assignment's subjects.
     */
    public HashSet getSubjects() {
        return getTarget().getSubjects();
    }
    
    /**
     * Compares the specified object with this assignment. Returns true if the
     * specified object is also an Assignment and the underlying policy element is also equal.
     * 
     * @param obj
     *            Object to be compared for equality with this assignment.
     * @return <code>true</code> if the specified Object is equal to this
     *         assignment.
     */
    public boolean equals(Object obj) {
        boolean result = false;
        if (obj instanceof Assignment)
        {
            result = super.equals(obj);
        }
        return result;
    }

    
}