/*
 * $Id: RBACPolicyElement.java,v 1.8 2005/11/03 21:15:03 suhrbier Exp $
 * Created on 15.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RBACPolicyElement.java,v $
 * Revision 1.8  2005/11/03 21:15:03  suhrbier
 * Add/Remove PermissionPolicy To Role
 *
 * Revision 1.7  2005/11/02 20:10:33  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.6  2005/10/31 16:25:35  suhrbier
 * *** empty log message ***
 *
 * Revision 1.5  2005/10/24 20:48:59  suhrbier
 * RoleAssignmentPolicyTest luft bis auf encode(). (target=null)
 *
 * Revision 1.4  2005/10/24 09:38:39  suhrbier
 * Assignment test ok
 *
 * Revision 1.3  2005/10/18 18:15:54  suhrbier
 * RoleAssignment Umstellung abgeschlossen!
 *
 * Revision 1.2  2005/10/17 21:07:20  suhrbier
 * Assignment auf RBACTarget umgestellt.
 *
 * Revision 1.1  2005/10/15 18:00:50  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.net.URI;

import com.sun.xacml.ParsingException;
import com.sun.xacml.Policy;
import com.sun.xacml.PolicySet;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Rule;
import com.sun.xacml.Target;

/**
 * Abstract base class for the RBACPolicyElements {@link Assignment Assignment},
 * {@link RoleAssignmentPolicy RoleAssignmentPolicy}and
 * {@link RoleAssignmentPolicySet RoleAssignmentPolicySet}.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.8 $
 * @since
 */

public abstract class RBACPolicyElement {

    /**
     * Identifier of this element.
     */
    private Identifier identifier = null;

    /**
     * Target of this element.
     */
    private RBACTarget target;

    protected RBACPolicyElement() {
    }

    /**
     * Create an RBACPolicyElement.
     * 
     * @param id
     *            Identifier of this element (domain:assignment:label).
     * @throws ParsingException
     *             Indicates a malformed identifier representation.
     */
    public RBACPolicyElement(URI id) throws ParsingException {
        setIdentifier(id);
    }

    /**
     * Create an RBACPolicyElement.
     * 
     * @param id
     *            Identifier of this element (domain:assignment:label).
     */
    public RBACPolicyElement(Identifier id) {
        setIdentifier(id);
    }

    /**
     * Create an RBACPolicyElement from its PolicyTreeElement representation.
     * 
     * @param element
     *            The element's PolicyTreeElement representation.
     * @throws ParsingException
     */
    public RBACPolicyElement(PolicyTreeElement element) throws ParsingException {
        parse(element);
    }

    abstract public void parse(PolicyTreeElement element)
            throws ParsingException;

    /**
     * Allows subclass specific parsing of the policy element's
     * {@link Target Target}to an {@link RBACTarget RBACTarget}. For that, it
     * must be overwritten by subclasses.
     * The default implementation simply parses the target using the {@link RBACTarget#parse(Target) RBACTarget.parse(Target)} method. if the <code>target</code> is not <code>null</code>.
     * @param target
     *            The target to parse.
     * @throws ParsingException
     *             Signals errors during the parsing process.
     */
    public void parse(Target target) throws ParsingException {
        if(getTarget()!=null)
            getTarget().parse(target);
        else
            throw new ParsingException("Element's target is null.");
    }

    /**
     * Returns the {@link PolicyTreeElement PolicyTreeElement} representation of this policy element.
     * Should be one of <li>{@link Rule Rule}</li>, <li>{@link Policy Policy}</li> or <li>{@link PolicySet PolicySet}</li> 
     * 
     * @return The policy element's {@link PolicyTreeElement PolicyTreeElement} representation.
     */
    abstract public PolicyTreeElement toPolicyTreeElement();

    /**
     * Adds another policy element to this element.
     * Shall permit to update/add elements to the target.
     * @param element The element to add.
     * @return <code>true</code> if the element has been added succesfully,
     *         <code>false</code> otherwise.
     */
    abstract public boolean add(RBACPolicyElement element);
    /**
     * Removes another policy element from this element.
     * Shall permit to update/remove elements from the target.
     * @param element The element to remove.
     * @return <code>true</code> if the element has been added succesfully,
     *         <code>false</code> otherwise.
     */
    abstract public boolean remove(RBACPolicyElement element);

    /**
     * Sets the identifier of this element.
     * 
     * @param id
     *            The element's identifier.
     * @throws ParsingException
     *             Indicates a malformed identifier representation.
     */
    public void setIdentifier(URI id) throws ParsingException {
        this.identifier = Identifier.getInstance(id);
    }

    /**
     * Sets the identifier of this element.
     * 
     * @param id
     *            The element's identifier.
     */
    public void setIdentifier(Identifier id) {
        this.identifier = id;
    }

    /**
     * Gets the identifier of this element.
     * 
     * @return The element's identifier.
     */
    public Identifier getIdentifier() {
        return identifier;
    }

    /**
     * Gets the target of this element.
     * 
     * @return Returns the target.
     */
    public RBACTarget getTarget() {
        return target;
    }

    /**
     * Sets target of this element.
     * 
     * @param target
     *            The target to set.
     */
    public void setTarget(RBACTarget target) {
        this.target = target;
    }

    /**
     * The hash code corresponds to the hash code of the assigment's identifier.
     * 
     * @see Object#hashCode()
     */
    public int hashCode() {
        return identifier.hashCode();
    }

    /**
     * Compares the specified object with this assignment. Returns true if the
     * specified object is also a RBACTarget, both identifiers and both targets are equal.
     * 
     * @param obj
     *            Object to be compared for equality with this element.
     * @return <code>true</code> if the specified Object is equal to this
     *         element.
     */
    public boolean equals(Object obj) {
        boolean result = false;
        if (obj instanceof RBACPolicyElement)
        {
            result = identifier!=null?identifier.equals(((RBACPolicyElement) obj).getIdentifier()):((RBACPolicyElement) obj).getIdentifier()==null;
            result &= target!=null?target.equals(((RBACPolicyElement) obj).getTarget()):((RBACPolicyElement) obj).getTarget()==null;
        }
        return result;
    }
    public String toString()
    {
        return getIdentifier().getLabel();
    }
}
