/*
 * $Id: RBACPolicySet.java,v 1.6 2005/12/19 19:11:05 suhrbier Exp $
 * Created on 15.10.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RBACPolicySet.java,v $
 * Revision 1.6  2005/12/19 19:11:05  suhrbier
 * Version ready to be delivered to BGBM.
 *
 * Revision 1.5  2005/11/07 21:02:36  suhrbier
 * removeUserToRole test completed.
 *
 * Revision 1.4  2005/11/03 21:15:03  suhrbier
 * Add/Remove PermissionPolicy To Role
 *
 * Revision 1.3  2005/11/02 20:10:33  suhrbier
 * RolePolicySet ready. Start with RoleManager for Permissions.
 *
 * Revision 1.2  2005/10/31 16:25:35  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2005/10/15 18:00:50  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.profile.rbac;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.Iterator;

import nbi.util.ReflectionUtil;
import nbi.xmlsec.xacml.builder.PolicySetBuilder;

import com.sun.xacml.AbstractPolicy;
import com.sun.xacml.ParsingException;
import com.sun.xacml.Policy;
import com.sun.xacml.PolicyReference;
import com.sun.xacml.PolicySet;
import com.sun.xacml.PolicyTreeElement;
import com.sun.xacml.Target;
import com.sun.xacml.combine.PermitOverridesPolicyAlg;

/**
 * Base class for all RBACPolicySets.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.6 $
 * @since
 */

public class RBACPolicySet extends RBACPolicy {
    /**
     * Policy Representation Unknown.
     */
    public static final int REPRESENT_POLICIES_AS_NONE = 0;

    /**
     * Policy Representation Policy Reference.
     */
    public static final int REPRESENT_POLICIES_AS_POLICYREF = 1;

    /**
     * Policy Representation Policy.
     */
    public static final int REPRESENT_POLICIES_AS_POLICY = 2;

    /**
     * The policy representation type.
     */
    private int policyRepresentationType = 0;

    protected RBACPolicySet() {
    }

    /**
     * Create an RBACPolicySet.
     * 
     * @param id
     *            Identifier of this element (domain:assignment:label).
     * @param type
     *            Type of the policyRepresentation
     *            {@link RBACPolicySet#REPRESENT_POLICIES_AS_POLICYREF REPRESENT_POLICIES_AS_POLICYREF}
     *            or
     *            {@link RBACPolicySet#REPRESENT_POLICIES_AS_POLICYREF REPRESENT_POLICIES_AS_POLICY}.
     * @throws ParsingException
     *             Indicates a malformed identifier representation.
     */
    public RBACPolicySet(URI id, int type) throws ParsingException {
        super(id);
        setPolicyRepresentationType(type);
    }

    /**
     * Create an RBACPolicyElement.
     * 
     * @param id
     *            Identifier of this element (domain:assignment:label).
     * @param type
     *            Type of the policyRepresentation
     *            {@link RBACPolicySet#REPRESENT_POLICIES_AS_POLICYREF REPRESENT_POLICIES_AS_POLICYREF}
     *            or
     *            {@link RBACPolicySet#REPRESENT_POLICIES_AS_POLICYREF REPRESENT_POLICIES_AS_POLICY}.
     */
    public RBACPolicySet(Identifier id, int type) {
        super(id);
        setPolicyRepresentationType(type);
    }

    /**
     * Default method to parse policy sets.
     * <li>Verifies whether the parsed element is an policy set</li>
     * <li>Sets the identifier of the policy set</li>
     * <li>Calls {@link #parse(Target) parse(Target)}.</li>
     * <li>Adds each child element of the policy set according to its class
     * type {@link Policy Policy},{@link PolicyReference PolicyReference}or
     * {@link PolicySet PolicySet}, .</li>
     * <li>Sets the policy representation type according to the class type of
     * the first child element</li>
     * According to the class type of the child element, an instance of the
     * related class (<code>policyClass</code>,
     * <code>policyReferenceClass</code> or <code>policySetClass</code>) is
     * created with the child element as single parameter. This instance is
     * added to the set of policy elements. If the related class parameter is
     * <code>null</code>, then a {@link ParsingException ParsingException}is
     * thrown when the related child element type is parsed.
     * 
     * @param element
     *            The policy set to parse.
     * @param policyClass
     *            Class to be instantiated for child elements instances of the
     *            (@link Policy Policy} class.
     * 
     * @param policyReferenceClass
     *            Class to be instantiated for child elements instances of the
     *            (@link PolicyReference PolicyReference} class.
     * @param policySetClass
     *            Class to be instantiated for child elements instances of the
     *            (@link PolicySet PolicySet} class.
     * @throws ParsingException
     *             If some parsing error occurs.
     */
    protected void parse(PolicyTreeElement element, Class policyClass,
            Class policyReferenceClass, Class policySetClass)
            throws ParsingException {
        if (element == null)
            throw new ParsingException("Tree element is null");

        if ((element instanceof PolicySet) || (element instanceof PolicyReference)) {
            AbstractPolicy policySet = (AbstractPolicy)element;
            setIdentifier(policySet.getId());
            parse(policySet.getTarget());
            Iterator policyIt = policySet.getChildren().iterator();
            while (policyIt.hasNext()) {
                AbstractPolicy polObj = (AbstractPolicy) policyIt.next();
                try {
                    if (polObj instanceof Policy) {
                        if (policyClass != null) {
                            add((RBACPolicyElement) ReflectionUtil.getInstance(
                                    policyClass.getName(), Policy.class,
                                    (Policy) polObj));
                            if (getPolicyRepresentationType() == REPRESENT_POLICIES_AS_NONE)
                                setPolicyRepresentationType(REPRESENT_POLICIES_AS_POLICY);
                        } else
                            throw new ParsingException(
                                    "Unexpected policy element of class type"
                                            + polObj.getClass().getName() + ".");
                    } else if (polObj instanceof PolicyReference) {
                        if (policyReferenceClass != null) {
                            add((RBACPolicyElement) ReflectionUtil.getInstance(
                                    policyReferenceClass.getName(),
                                    PolicyReference.class,
                                    (PolicyReference) polObj));
                            if (getPolicyRepresentationType() == REPRESENT_POLICIES_AS_NONE)
                                setPolicyRepresentationType(REPRESENT_POLICIES_AS_POLICYREF);
                        } else
                            throw new ParsingException(
                                    "Unexpected policy element of class type"
                                            + polObj.getClass().getName() + ".");
                    } else if (polObj instanceof PolicySet) {
                        if (policySetClass != null) {
                            add((RBACPolicyElement) ReflectionUtil.getInstance(
                                    policySetClass.getName(), PolicySet.class,
                                    (PolicySet) polObj));
                            if (getPolicyRepresentationType() == REPRESENT_POLICIES_AS_NONE)
                                setPolicyRepresentationType(REPRESENT_POLICIES_AS_POLICYREF);
                        } else
                            throw new ParsingException(
                                    "Unexpected policy element of class type"
                                            + polObj.getClass().getName() + ".");
                    }
                } catch (Exception e) {
                    logger.error(e.toString());
                }
            }
        } else
            throw new ParsingException("Tree element " + element.getId()
                    + " is not a policy set");
    }

    /**
     * Not implemented. Must be overwritten in subclasses.
     *  
     */
    public void parse(PolicyTreeElement element) throws ParsingException {
        parse(element, null,null,null);
    }

    /**
     * Default method to generate the {@link PolicySet PolicySet}representation
     * of this RBACPolicySet. Set the policy algorithm to
     * {@link PermitOverridesPolicyAlg PermitOverridesPolicyAlg}. For each
     * element of the policy set, adds the policy reference or policy
     * representation of the element to the policy set. The representation type
     * for policies is set according to the policy representation type. Policy
     * sets are represented as policy references.
     * 
     * @return The {@link PolicySet PolicySet}representation.
     */
    public PolicyTreeElement toPolicyTreeElement() {
        PolicySetBuilder policySetBuilder = new PolicySetBuilder(
                getIdentifier().toURI(), new PermitOverridesPolicyAlg());
        policySetBuilder.setTarget(getTarget().toTarget());
        if (elements != null) {
            Iterator it = elements.iterator();
            while (it.hasNext()) {
                RBACPolicy polObj = (RBACPolicy) it.next();
                if (polObj instanceof RBACPolicySet) {
                    policySetBuilder.add(polObj.toPolicyRerence());
                } else if (polObj instanceof RBACPolicy) {
                    if (getPolicyRepresentationType() == REPRESENT_POLICIES_AS_POLICY)
                        policySetBuilder.add((Policy) polObj
                                .toPolicyTreeElement());
                    else
                        policySetBuilder.add(polObj.toPolicyRerence());
                }
            }
        }
        return (PolicySet) policySetBuilder.getInstance();
    }

    /**
     * Returns the {@link PolicyReference PolicyReference} representation of
     * this policy set. The default type of the policy reference is set to
     * {@link PolicyReference#POLICYSET_REFERENCE PolicyReference.POLICYSET_REFERENCE}).
     * @return The policy element's {@link PolicyReference PolicyReference}
     *         representation with type set to {@link PolicyReference#POLICYSET_REFERENCE PolicyReference.POLICY_REFERENCE}).
     */
    public PolicyReference toPolicyRerence() {
        return toPolicyRerence(PolicyReference.POLICYSET_REFERENCE);
    }
    

    /**
     * Encodes this PolicySet and all included RBACPolicy or RBACPolicySet
     * elements into its XML representation and writes them to the file path
     * using the <code>path</code> parameter as parent directory and the
     * identifier's file representation as file name. Creates directories if
     * missing.
     * 
     * @param path
     *            Base directory path to the policies.
     */
    public void encode(File path) throws IOException {
        super.encode(path);
        if(elements!=null)
        {
            Iterator elemIt = elements.iterator();
            while(elemIt.hasNext())
            {
                ((RBACPolicy)elemIt.next()).encode(path);
            }
        }
    }
    /**
     * Gets the policyRepresentationType
     * 
     * @return Returns the policyRepresentationType.
     */
    public int getPolicyRepresentationType() {
        return policyRepresentationType;
    }

    /**
     * Sets policyRepresentationType
     * 
     * @param policyRepresentationType
     *            The policyRepresentationType to set.
     */
    public void setPolicyRepresentationType(int policyRepresentationType) {
        this.policyRepresentationType = policyRepresentationType;
    }
}
