/*
 * $Id: PermissionMatchBuilder.java,v 1.2 2005/12/19 19:11:04 suhrbier Exp $
 * Created on 04.11.2005
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: PermissionMatchBuilder.java,v $
 * Revision 1.2  2005/12/19 19:11:04  suhrbier
 * Version ready to be delivered to BGBM.
 *
 * Revision 1.1  2005/11/04 17:52:36  suhrbier
 * Permission add mit Targets funzt.
 *
 */
package nbi.xmlsec.xacml.profile.rbac.builder;

import java.net.URI;

import javax.security.auth.x500.X500Principal;

import nbi.xmlsec.xacml.builder.ActionAttributeDesignatorBuilder;
import nbi.xmlsec.xacml.builder.AttributeDesignatorBuilder;
import nbi.xmlsec.xacml.builder.ResourceAttributeDesignatorBuilder;
import nbi.xmlsec.xacml.builder.TargetMatchBuilder;

import com.sun.xacml.TargetMatch;
import com.sun.xacml.UnknownIdentifierException;
import com.sun.xacml.attr.AnyURIAttribute;
import com.sun.xacml.attr.AttributeDesignator;
import com.sun.xacml.attr.AttributeValue;
import com.sun.xacml.attr.StringAttribute;
import com.sun.xacml.attr.X500NameAttribute;
import com.sun.xacml.cond.EqualFunction;
import com.sun.xacml.cond.FunctionTypeException;
import com.sun.xacml.cond.MatchFunction;

/**
 * This class serves to create specialised TargetMatches for Permissions from
 * target scheme descriptions(datatype-matchfunc[value]). The following functions are supported:
 * <li>string-equal</li>
 * <li>anyuri-equal</li>
 * <li>x500Name-equal</li>
 * <li>string-match</li>
 * <li>x500Name-match</li>
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.2 $
 * @since
 */

public class PermissionMatchBuilder extends TargetMatchBuilder {

    /**
     * Creates a PermissionMatchBuilder based on the type target match
     * (SubjectMatch, ResourceMatch, or ActionMatch) to build.
     * 
     * @param type
     *            type of the target match.
     */
    public PermissionMatchBuilder(int type) {
        super(type);
    }

    /**
     * Retrieves the matching SunXACML function id for Permissions. Currently,
     * the following functions are supported:
     * <li>string-equal</li>
     * <li>anyuri-equal</li>
     * <li>x500Name-equal</li>
     * <li>string-match</li>
     * <li>x500Name-match</li>
     * 
     * @param dataType
     *            datatype of the match function.
     * @param function
     *            name of the match function.
     * @return The SunXACML identifier of the match function.
     */
    public String getFunctionId(String dataType, String function) {
        String result = null;
        if ("equal".equals(function)) {
            if ("string".equals(dataType))
                result = EqualFunction.NAME_STRING_EQUAL;
            else if ("anyuri".equals(dataType))
                result = EqualFunction.NAME_ANYURI_EQUAL;
            else if ("x500Name".equals(dataType))
                result = EqualFunction.NAME_X500NAME_EQUAL;
        } else if ("match".equals(function)) {
            if ("string".equals(dataType))
                result = MatchFunction.NAME_REGEXP_STRING_MATCH;
            else if ("x500Name".equals(dataType))
                result = MatchFunction.NAME_X500NAME_MATCH;
        }
        return result;
    }

    /**
     * Retrieves the SunXACML Attribute identifier for a given datatype: The
     * following datatypes are supported:
     * <li>string</li>
     * <li>anyuri</li>
     * <li>x500Name</li>
     * 
     * @param dataType
     *            data type to search for.
     * @return The corresponding XACML-Attribute-identfier of the datatype.
     */
    public String getDataTypeId(String dataType) {
        String result = null;
        if ("string".equals(dataType))
            result = StringAttribute.identifier;
        else if ("anyuri".equals(dataType))
            result = AnyURIAttribute.identifier;
        else if ("x500Name".equals(dataType))
            result = X500NameAttribute.identifier;
        return result;
    }

    /**
     * Parses and sets the {@link TargetMatchBuilder TargetMatchBuilder}
     * attribute from a target syntax scheme description. The syntax scheme
     * description must match the following scheme: datatype-matchfunc[value].
     * 
     * @param obj
     *            target syntax scheme description
     * @throws UnknownIdentifierException
     *             Unknown identifier type used
     * @throws FunctionTypeException
     *             Unknown function type used
     */
    public void setAttribute(String obj) throws UnknownIdentifierException,
            FunctionTypeException {
        int startInd = 0, endInd = obj.indexOf("-", startInd);
        String dataType = obj.substring(startInd, endInd);
        startInd = endInd + 1;
        endInd = obj.indexOf("[", startInd);
        String matchfunc = obj.substring(startInd, endInd);
        startInd = endInd + 1;
        endInd = obj.indexOf("]", startInd);
        String value = obj.substring(startInd, endInd);

        setMatchFunction(getFunctionId(dataType, matchfunc));
        AttributeDesignatorBuilder evalBuilder = null;
        switch (type) {
        case TargetMatch.RESOURCE:
            evalBuilder = new ResourceAttributeDesignatorBuilder();
            break;
        case TargetMatch.ACTION:
            evalBuilder = new ActionAttributeDesignatorBuilder();
            break;
        }
        evalBuilder.setType(getDataTypeId(dataType));
        setMatchEvaluatable((AttributeDesignator) evalBuilder.getInstance());
        super.setAttribute(value);
    }

    /**
     * Create the appropriate attribute match value for the evaluation function.
     * 
     * @see nbi.xmlsec.xacml.builder.TargetMatchBuilder#createMatchValue()
     */
    public AttributeValue createMatchValue() {
        AttributeValue result = null;
        if (StringAttribute.identifier.equals(eval.getType().toString()))
            result = new StringAttribute((String) attribute);
        else if (AnyURIAttribute.identifier.equals(eval.getType().toString()))
            result = new AnyURIAttribute((URI) attribute);
        else if (X500NameAttribute.identifier.equals(eval.getType().toString()))
            result = new X500NameAttribute((X500Principal) attribute);

        return result;

    }
}
