/*
 * $Id: RBACRequestCtxBuilder.java,v 1.4 2006/07/24 17:31:09 suhrbier Exp $
 * Created on 19.04.2006
 *
 ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * "The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * The Original Code is XML Security Services for BioCASE Provider Software.
 * 
 * The Initial Developer of the Original Code is 
 * Lutz Suhrbier (suhrbier@inf.fu-berlin.de)
 * Networked Information Systems
 * Department of Mathematics and Computer Science
 * Freie Universitt Berlin
 * Takustrasse 9
 * D-14195 Berlin.
 * 
 * Portions created by the Initial Developer are Copyright (C) 2004-2006
 * by the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s):
 *
 ***** END LICENSE BLOCK *****
 * 
 * $Log: RBACRequestCtxBuilder.java,v $
 * Revision 1.4  2006/07/24 17:31:09  suhrbier
 * update JavaDoc
 *
 * Revision 1.3  2006/04/24 19:50:19  suhrbier
 * *** empty log message ***
 *
 * Revision 1.2  2006/04/20 17:15:27  suhrbier
 * *** empty log message ***
 *
 * Revision 1.1  2006/04/19 15:31:20  suhrbier
 * *** empty log message ***
 *
 */
package nbi.xmlsec.xacml.profile.rbac.builder;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import nbi.xmlsec.xacml.builder.ActionAttributeDesignatorBuilder;
import nbi.xmlsec.xacml.builder.RequestCtxBuilder;
import nbi.xmlsec.xacml.builder.ResourceAttributeDesignatorBuilder;
import nbi.xmlsec.xacml.profile.rbac.RoleAssignmentPolicyIdentifier;
import nbi.xmlsec.xacml.profile.rbac.RolePolicySet;
import nbi.xmlsec.xacml.profile.rbac.RoleValueIdentifier;

import org.apache.log4j.Logger;

import com.sun.xacml.attr.AnyURIAttribute;
import com.sun.xacml.attr.StringAttribute;
import com.sun.xacml.ctx.Attribute;
import com.sun.xacml.ctx.Subject;

/**
 * Class to create RBAC specific RequestCtxs.
 * 
 * @author Lutz Suhrbier
 * @version $Revision: 1.4 $
 * @since
 */

public class RBACRequestCtxBuilder extends RequestCtxBuilder {

    static Logger logger = Logger.getLogger(RBACRequestCtxBuilder.class);

    /**
     * Default Constructor.
     */
    public RBACRequestCtxBuilder() {
        super();
    }

    /**
     * Adds a role to the subject part of a request context on a given
     * RoleAssignmentPolicyIdentifier. Uses the attribute id of the
     * RolePolicySet as subject id.
     * 
     * @param roleId
     *            The RoleAssignmentPolicyIdentifier
     * @return true if the role has been added succesfully.
     * @throws URISyntaxException
     */
    public boolean addRole(RoleAssignmentPolicyIdentifier roleId) {
        HashSet subjects = new HashSet();

        HashSet subjectAttrs = new HashSet();

        URI subjectId = null;
        try {
            subjectId = new URI(RolePolicySet.ATTRIBUTE_ID);
        } catch (URISyntaxException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        AnyURIAttribute subjectValue = new AnyURIAttribute(
                new RoleValueIdentifier(roleId.getDomain(), roleId.getLabel())
                        .toURI());
        subjectAttrs.add(new Attribute(subjectId, null, null, subjectValue));
        return add(new Subject(subjectAttrs));
    }

    /**
     * Adds a set of roles to the subject part of a request context using
     * {@link RBACRequestCtxBuilder#addRole(RoleAssignmentPolicyIdentifier) addRole(RoleAssignmentPolicyIdentifier)}.
     * 
     * @param roleIds
     *            A set of
     *            {@link RoleAssignmentPolicyIdentifier RoleAssignmentPolicyIdentifiers}.
     * @return true if the role has been added succesfully.
     * @throws URISyntaxException
     */
    public boolean addRoles(Set roleIds) {
        boolean ret = true;
        Iterator roleIt = roleIds.iterator();
        while (ret && roleIt.hasNext()) {
            RoleAssignmentPolicyIdentifier roleId = (RoleAssignmentPolicyIdentifier) roleIt
                    .next();
            ret = addRole(roleId);
        }
        return ret;
    }

    /**
     * Removes a role from the subject part of a request context on a given
     * RoleAssignmentPolicyIdentifier. Uses the attribute id of the
     * RolePolicySet as subject id.
     * 
     * @param roleId
     *            The RoleAssignmentPolicyIdentifier
     * @return true if the role has been removed succesfully.
     * @throws URISyntaxException
     */
    public boolean removeRole(RoleAssignmentPolicyIdentifier roleId) {
        HashSet subjects = new HashSet();

        HashSet subjectAttrs = new HashSet();
        URI subjectId = null;
        try {
            subjectId = new URI(RolePolicySet.ATTRIBUTE_ID);
        } catch (URISyntaxException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        AnyURIAttribute subjectValue = new AnyURIAttribute(
                new RoleValueIdentifier(roleId.getDomain(), roleId.getLabel())
                        .toURI());
        subjectAttrs.add(new Attribute(subjectId, null, null, subjectValue));
        return remove(new Subject(subjectAttrs));
    }

    /**
     * Adds the given resource string to the resource part of a request context.
     * Uses the attribute id of the ResourceAttributeDesignator as resource id.
     * 
     * @param resource
     *            The resource string.
     * @return true if the resource has been added succesfully.
     * @throws URISyntaxException
     */
    public boolean addResource(String resource) {
        URI resourceId = null;
        try {
            resourceId = new URI(ResourceAttributeDesignatorBuilder.attributeId);
        } catch (URISyntaxException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        StringAttribute resourceValue = new StringAttribute(resource);
        return addResource(new Attribute(resourceId, null, null, resourceValue));
    }

    /**
     * Removes the given resource string from the resource part of a request
     * context. Uses the attribute id of the ResourceAttributeDesignator as
     * resource id.
     * 
     * @param resource
     *            The resource string.
     * @return true if the resource has been removed succesfully.
     * @throws URISyntaxException
     */
    public boolean removeResource(String resource) {
        URI resourceId = null;
        try {
            resourceId = new URI(ResourceAttributeDesignatorBuilder.attributeId);
        } catch (URISyntaxException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        StringAttribute resourceValue = new StringAttribute(resource);
        return removeResource(new Attribute(resourceId, null, null,
                resourceValue));
    }

    /**
     * Adds the action part of a request context on a given reource String. Uses
     * the attribute id of the ActionAttributeDesignator as action id.
     * 
     * @param action
     *            The action string.
     * @return true if the action has been added succesfully.
     * @throws URISyntaxException
     */
    public boolean addAction(String action){
        URI actionId = null;
        try {
            actionId = new URI(ActionAttributeDesignatorBuilder.attributeId);
        } catch (URISyntaxException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        StringAttribute actionValue = new StringAttribute(action);
        return addAction(new Attribute(actionId, null, null, actionValue));
    }

    /**
     * Removes the given action string from the action part of a request
     * context. Uses the attribute id of the ActionAttributeDesignator as action
     * id.
     * 
     * @param action
     *            The action string.
     * @return true if the action has been removed succesfully.
     * @throws URISyntaxException
     */
    public boolean removeAction(String action){
        URI actionId = null;
        try {
            actionId = new URI(ActionAttributeDesignatorBuilder.attributeId);
        } catch (URISyntaxException e) {
            logger.fatal("The exception " + e + "should never happen!");
        }
        StringAttribute actionValue = new StringAttribute(action);
        return removeAction(new Attribute(actionId, null, null, actionValue));
    }
}
